@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2009 Sourcefire, Inc.
+ *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *  Author: Tomasz Kojm <tkojm@clamav.net>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -139,8 +139,8 @@ static int printconf(const char *name)
     for(i = 0; clam_options[i].owner; i++) {
 	cpt = &clam_options[i];
 	if(cpt->name && (cpt->owner & tool) && !(cpt->owner & OPT_DEPRECATED) && !(cpt->flags & 4)) {
-	    strncpy(buffer, cpt->description, 1024);
-	    buffer[1024] = 0;
+	    strncpy(buffer, cpt->description, sizeof(buffer)-1);
+	    buffer[sizeof(buffer)-1] = 0;
 	    tokens_count = cli_strtokenize(buffer, '\n', 128, tokens);
 	    printf("\n");
 	    for(j = 0; j < tokens_count; j++)
@@ -181,8 +181,8 @@ static int printconf(const char *name)
 	    }
 
 	    if(cpt->suggested && strchr(cpt->suggested, '\n')) {
-		strncpy(buffer, cpt->suggested, 1024);
-		buffer[1024] = 0;
+		strncpy(buffer, cpt->suggested, sizeof(buffer)-1);
+		buffer[sizeof(buffer)-1] = 0;
 		tokens_count = cli_strtokenize(buffer, '\n', 128, tokens);
 		for(j = 0; j < tokens_count; j++)
 		    printf("#%s %s\n", cpt->name, tokens[j]);
@@ -230,10 +230,14 @@ static void print_platform(struct cli_environment *env)
 #else
     /* e.g. Solaris */
     if (!access("/etc/release", R_OK)) {
-	char buf[1024];
-	FILE *f = fopen("/etc/release", "r");
-	fgets(buf, sizeof(buf), f);
-	printf("Full OS version: %s", buf);
+        char buf[1024];
+        FILE *f = fopen("/etc/release", "r");
+
+        if (f) {
+            fgets(buf, sizeof(buf), f);
+            printf("Full OS version: %s", buf);
+            fclose(f);
+        }
     }
 #endif
 
@@ -326,6 +330,7 @@ static void print_dbs(const char *dir)
 		dbfile = (char *) malloc(strlen(dent->d_name) + strlen(dir) + 2);
 		if(!dbfile) {
 		    printf("print_dbs: Can't allocate memory for dbfile\n");
+		    closedir(dd);
 		    return;
 		}
 		sprintf(dbfile, "%s"PATHSEP"%s", dir, dent->d_name);
@@ -366,7 +371,6 @@ int main(int argc, char **argv)
 	unsigned int i, j;
 	struct cli_environment env;
 
-
     opts = optparse(NULL, argc, argv, 1, OPT_CLAMCONF, 0, NULL);
     if(!opts) {
 	printf("ERROR: Can't parse command line options\n");
@@ -392,6 +396,7 @@ int main(int argc, char **argv)
     }
 
     dbdir[0] = 0;
+    clamd_dbdir[0] = 0;
     dir = optget(opts, "config-dir")->strarg;
     printf("Checking configuration files in %s\n", dir);
     for(i = 0; cfgfile[i].name; i++) {
