@@ -1,5 +1,7 @@
 /*
- *  Copyright (C) 2002 Tomasz Kojm <tkojm@clamav.net>
+ *  Copyright (C) 2007-2009 Sourcefire, Inc.
+ *
+ *  Authors: Tomasz Kojm
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -19,8 +21,8 @@
 #ifndef __LOCALSERVER_H
 #define __LOCALSERVER_H
 
-#include "shared/cfgparser.h"
+#include "shared/optparser.h"
 
-int localserver(const struct cfgstruct *copt);
+int localserver(const struct optstruct *opts);
 
 #endif
