@@ -86,8 +86,8 @@ void detstats_clear(void);
 void detstats_add(const char *virname, const char *fname, unsigned int fsize, const char *md5);
 void detstats_print(int desc, char term);
 
-#ifdef CLAMUKO
-int clamuko_checkowner(int pid, const struct optstruct *opts);
+#ifdef FANOTIFY
+int fan_checkowner(int pid, const struct optstruct *opts);
 #endif
 
 #endif
