@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2007-2012 Sourcefire, Inc.
+ *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
  *
@@ -249,7 +249,7 @@ static void scanfile(const char *filename, struct cl_engine *engine, const struc
     }
 
     /* argh, don't scan /proc files */
-    if(STAT(filename, &sb) != -1) {
+    if(CLAMSTAT(filename, &sb) != -1) {
 #ifdef C_LINUX
 	if(procdev && sb.st_dev == procdev) {
 	    if(!printinfected)
@@ -412,7 +412,7 @@ static void scandirs(const char *dirname, struct cl_engine *engine, const struct
 			    if(dirlnk != 2 && filelnk != 2) {
 				if(!printinfected)
 				    logg("%s: Symbolic link\n", fname);
-			    } else if(STAT(fname, &sb) != -1) {
+			    } else if(CLAMSTAT(fname, &sb) != -1) {
 				if(S_ISREG(sb.st_mode) && filelnk == 2) {
 				    scanfile(fname, engine, opts, options);
 				} else if(S_ISDIR(sb.st_mode) && dirlnk == 2) {
@@ -631,6 +631,9 @@ int scanmanager(const struct optstruct *opts)
     if(optget(opts, "leave-temps")->enabled)
 	cl_engine_set_num(engine, CL_ENGINE_KEEPTMP, 1);
 
+    if(optget(opts, "force-to-disk")->enabled)
+	cl_engine_set_num(engine, CL_ENGINE_FORCETODISK, 1);
+
     if(optget(opts, "bytecode-unsigned")->enabled)
 	dboptions |= CL_DB_BYTECODE_UNSIGNED;
 
@@ -884,7 +887,7 @@ int scanmanager(const struct optstruct *opts)
 
 #ifdef C_LINUX
     procdev = (dev_t) 0;
-    if(STAT("/proc", &sb) != -1 && !sb.st_size)
+    if(CLAMSTAT("/proc", &sb) != -1 && !sb.st_size)
 	procdev = sb.st_dev;
 #endif
 
@@ -895,7 +898,7 @@ int scanmanager(const struct optstruct *opts)
 	    logg("!Can't get absolute pathname of current working directory\n");
 	    ret = 2;
 	} else {
-	    STAT(cwd, &sb);
+	    CLAMSTAT(cwd, &sb);
 	    scandirs(cwd, engine, opts, options, 1, sb.st_dev);
 	}
 
@@ -908,8 +911,8 @@ int scanmanager(const struct optstruct *opts)
 
 	while((filename = filelist(opts, &ret)) && (file = strdup(filename))) {
 	    if(LSTAT(file, &sb) == -1) {
-		logg("^%s: Can't access file\n", file);
 		perror(file);
+		logg("^%s: Can't access file\n", file);
 		ret = 2;
 	    } else {
 		for(i = strlen(file) - 1; i > 0; i--) {
@@ -923,7 +926,7 @@ int scanmanager(const struct optstruct *opts)
 		    if(dirlnk == 0 && filelnk == 0) {
 			if(!printinfected)
 			    logg("%s: Symbolic link\n", file);
-		    } else if(STAT(file, &sb) != -1) {
+		    } else if(CLAMSTAT(file, &sb) != -1) {
 			if(S_ISREG(sb.st_mode) && filelnk) {
 			    scanfile(file, engine, opts, options);
 			} else if(S_ISDIR(sb.st_mode) && dirlnk) {
