@@ -116,6 +116,7 @@ static void MT_decrypt(uint8_t *buf, unsigned int size, uint32_t seed) {
   for(i=1; i<624; i++)
     mt[i] = i+0x6c078965*((mt[i-1]>>30)^mt[i-1]);
   MT.items = 1;
+  MT.next = MT.mt;
 
   while(size--)
     *buf++ ^= MT_getnext(&MT);
@@ -176,7 +177,7 @@ static uint32_t getbits(struct UNP *UNP, uint32_t size) {
 *********************/
 
 
-static int ea05(cli_ctx *ctx, uint8_t *base, char *tmpd) {
+static int ea05(cli_ctx *ctx, const uint8_t *base, char *tmpd) {
   uint8_t b[300], comp;
   uint32_t s, m4sum=0;
   int i, ret;
@@ -378,7 +379,11 @@ static int ea05(cli_ctx *ctx, uint8_t *base, char *tmpd) {
       cli_dbgmsg("autoit: file extracted to %s\n", tempfile);
     else 
       cli_dbgmsg("autoit: file successfully extracted\n");
-    lseek(i, 0, SEEK_SET);
+    if (lseek(i, 0, SEEK_SET) == -1) {
+        cli_dbgmsg("autoit: call to lseek() has failed\n");
+        close(i);
+        return CL_ESEEK;
+    }
     if(cli_magic_scandesc(i, ctx) == CL_VIRUS) {
       close(i);
       if(!ctx->engine->keeptmp)
@@ -480,7 +485,7 @@ static void LAME_decrypt (uint8_t *cypher, uint32_t size, uint16_t seed) {
  autoit3 EA06 handler 
 *********************/
 
-static int ea06(cli_ctx *ctx, uint8_t *base, char *tmpd) {
+static int ea06(cli_ctx *ctx, const uint8_t *base, char *tmpd) {
   uint8_t b[600], comp, script, *buf;
   uint32_t s;
   int i, ret;
@@ -895,7 +900,11 @@ static int ea06(cli_ctx *ctx, uint8_t *base, char *tmpd) {
       cli_dbgmsg("autoit: %s extracted to %s\n", (script)?"script":"file", tempfile);
     else 
       cli_dbgmsg("autoit: %s successfully extracted\n", (script)?"script":"file");
-    lseek(i, 0, SEEK_SET);
+    if (lseek(i, 0, SEEK_SET) == -1) {
+        cli_dbgmsg("autoit: call to lseek() has failed\n");
+        close(i);
+        return CL_ESEEK;
+    }
     if(cli_magic_scandesc(i, ctx) == CL_VIRUS) {
       close(i);
       if(!ctx->engine->keeptmp) 
@@ -916,7 +925,7 @@ static int ea06(cli_ctx *ctx, uint8_t *base, char *tmpd) {
 *********************/
 
 int cli_scanautoit(cli_ctx *ctx, off_t offset) {
-  uint8_t *version;
+  const uint8_t *version;
   int r;
   char *tmpd;
   fmap_t *map = *ctx->fmap;
