@@ -22,5 +22,5 @@
 #define __AUTOIT_H
 
 #include "others.h"
-int cli_scanautoit(int desc, cli_ctx *ctx, off_t offset);
+int cli_scanautoit(cli_ctx *ctx, off_t offset);
 #endif
