@@ -48,7 +48,8 @@ static const uint8_t hqxtbl[] = {
 
 int cli_binhex(cli_ctx *ctx) {
     fmap_t *map = *ctx->fmap;
-    uint8_t *encoded, decoded[BUFSIZ], spare_bits, last_byte=0, this_byte, offset=0;
+    const uint8_t *encoded;
+    uint8_t decoded[BUFSIZ], spare_bits, last_byte=0, this_byte, offset=0;
     size_t enc_done=0, enc_todo=map->len;
     unsigned int dec_done=0, chunksz = 0, chunkoff=0;
     uint32_t datalen, reslen;
@@ -69,6 +70,8 @@ int cli_binhex(cli_ctx *ctx) {
 	return ret;
     }
 
+    memset(decoded, 0, 24);
+
     while(1) {
 	uint8_t b;
 	if(!enc_todo || dec_done >= BH_FLUSH_SZ) {
@@ -106,7 +109,11 @@ int cli_binhex(cli_ctx *ctx) {
 		}
 		if(!datalen) {
 		    write_phase++;
-		    lseek(datafd, 0, SEEK_SET);
+		    if (lseek(datafd, 0, SEEK_SET) == -1) {
+                cli_dbgmsg("cli_binhex: call to lseek() has failed\n");
+                ret = CL_ESEEK;
+                break;
+            }
 		    ret = cli_magic_scandesc(datafd, ctx);
 		    if(ret == CL_VIRUS) break;
 		}
@@ -148,7 +155,11 @@ int cli_binhex(cli_ctx *ctx) {
 		    break;
 		}
 		if(!reslen) {
-		    lseek(resfd, 0, SEEK_SET);
+		    if (lseek(resfd, 0, SEEK_SET) == -1) {
+                cli_dbgmsg("cli_binhex: call to lseek() has failed\n");
+                ret = CL_ESEEK;
+                break;
+            }
 		    ret = cli_magic_scandesc(resfd, ctx);
 		    break;
 		}
@@ -156,11 +167,19 @@ int cli_binhex(cli_ctx *ctx) {
 	    if(!enc_todo) {
 		if(write_phase == IN_DATA) {
 		    cli_dbgmsg("cli_binhex: scanning partially extracted data fork\n");
-		    lseek(datafd, 0, SEEK_SET);
+		    if (lseek(datafd, 0, SEEK_SET) == -1) {
+                cli_dbgmsg("cli_binhex: call to lseek() has failed\n");
+                ret = CL_ESEEK;
+                break;
+            }
 		    ret = cli_magic_scandesc(datafd, ctx);
 		} else if(write_phase == IN_RES) {
 		    cli_dbgmsg("cli_binhex: scanning partially extracted resource fork\n");
-		    lseek(resfd, 0, SEEK_SET);
+		    if (lseek(resfd, 0, SEEK_SET) == -1) {
+                cli_dbgmsg("cli_binhex: call to lseek() has failed\n");
+                ret = CL_ESEEK;
+                break;
+            }
 		    ret = cli_magic_scandesc(resfd, ctx);
 		}
 		break;
