@@ -575,7 +575,7 @@ fileblobAddData(fileblob *fb, const unsigned char *data, size_t len)
 				fb->bytes_scanned += (unsigned long)len;
 				
 				if((len > 5) && cli_updatelimits(ctx, len)==CL_CLEAN && (cli_scanbuff(data, (unsigned int)len, 0, ctx->virname, ctx->engine, CL_TYPE_BINARY_DATA, NULL) == CL_VIRUS)) {
-					cli_dbgmsg("fileblobAddData: found %s\n", *ctx->virname);
+				    cli_dbgmsg("fileblobAddData: found %s\n", cli_get_last_virus_str(ctx->virname));
 					fb->isInfected = 1;
 				}
 			}
@@ -615,7 +615,7 @@ int
 fileblobScan(const fileblob *fb)
 {
 	int rc;
-	struct stat sb;
+	STATBUF sb;
 
 	if(fb->isInfected)
 		return CL_VIRUS;
@@ -632,7 +632,7 @@ fileblobScan(const fileblob *fb)
 
 	fflush(fb->fp);
 	lseek(fb->fd, 0, SEEK_SET);
-	fstat(fb->fd, &sb);
+	FSTAT(fb->fd, &sb);
 	if(cli_matchmeta(fb->ctx, fb->b.name, sb.st_size, sb.st_size, 0, 0, 0, NULL) == CL_VIRUS)
 	    return CL_VIRUS;
 
