@@ -1,7 +1,7 @@
 /*
  *  Load, and verify ClamAV bytecode.
  *
- *  Copyright (C) 2009-2010 Sourcefire, Inc.
+ *  Copyright (C) 2009-2013 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
  *
@@ -40,6 +40,13 @@
 #include "builtin_bytecodes.h"
 #include <string.h>
 
+#define MAX_BC 64
+#define BC_EVENTS_PER_SIG 2
+#define MAX_BC_SIGEVENT_ID MAX_BC*BC_EVENTS_PER_SIG
+
+cli_events_t * g_sigevents = NULL;
+unsigned int g_sigid;
+
 /* dummy values */
 static const uint32_t nomatch[64] = {
     0xdeadbeef, 0xdeaddead, 0xbeefdead, 0xdeaddead, 0xdeadbeef, 0, 0, 0,
@@ -82,7 +89,7 @@ static void context_safe(struct cli_bc_ctx *ctx)
     if (!ctx->hooks.match_counts)
 	ctx->hooks.match_counts = nomatch;
     if (!ctx->hooks.match_offsets)
-	ctx->hooks.match_counts = nooffsets;
+	ctx->hooks.match_offsets = nooffsets;
     if (!ctx->hooks.filesize)
 	ctx->hooks.filesize = &nofilesize;
     if (!ctx->hooks.pedata)
@@ -93,6 +100,10 @@ static int cli_bytecode_context_reset(struct cli_bc_ctx *ctx);
 struct cli_bc_ctx *cli_bytecode_context_alloc(void)
 {
     struct cli_bc_ctx *ctx = cli_calloc(1, sizeof(*ctx));
+    if (!ctx) {
+        cli_errmsg("Out of memory allocating cli_bytecode_context_reset\n");
+        return NULL;
+    }
     ctx->bytecode_timeout = 60000;
     cli_bytecode_context_reset(ctx);
     return ctx;
@@ -150,8 +161,10 @@ static int cli_bytecode_context_reset(struct cli_bc_ctx *ctx)
 	    if(fd >= 0) {
 		ret = cli_scandesc(fd, cctx, CL_TYPE_HTML, 0, NULL, AC_SCAN_VIR, NULL);
 		if (ret == CL_CLEAN) {
-		    lseek(fd, 0, SEEK_SET);
-		    ret = cli_scandesc(fd, cctx, CL_TYPE_TEXT_ASCII, 0, NULL, AC_SCAN_VIR, NULL);
+		    if (lseek(fd, 0, SEEK_SET) == -1)
+                cli_dbgmsg("cli_bytecode: call to lseek() has failed\n");
+            else
+                ret = cli_scandesc(fd, cctx, CL_TYPE_TEXT_ASCII, 0, NULL, AC_SCAN_VIR, NULL);
 		}
 		close(fd);
 	    }
@@ -513,6 +526,7 @@ static inline char *readData(const unsigned char *p, unsigned *off, unsigned len
 	if (UNLIKELY((v0&0xf0) != 0x60 || (v1&0xf0) != 0x60)) {
 	    cli_errmsg("Invalid data part: %c%c\n", v0, v1);
 	    *ok = 0;
+	    free(dat);
 	    return 0;
 	}
 	*q++ = (v0&0xf) | ((v1&0xf) << 4);
@@ -637,7 +651,8 @@ static int parseLSig(struct cli_bc *bc, char *buffer)
 	vnames = strchr(vend, '{');
     } else {
 	/* Not a logical signature, but we still have a virusname */
-	bc->lsig = NULL;
+	bc->hook_name = cli_strdup(buffer);
+	bc->lsig = NULL; 
     }
 
     return CL_SUCCESS;
@@ -884,8 +899,10 @@ static int parseApis(struct cli_bc *bc, unsigned char *buffer)
 	}
 	/* don't need the name anymore */
 	free(name);
-	if (!ok)
+	if (!ok) {
+	    free(apity2ty); /* free temporary map */
 	    return CL_EMALFDB;
+	}
 
 	/* APIcall is valid */
 	cli_bitset_set(bc->uses_apis, id);
@@ -1373,8 +1390,10 @@ static int parseBB(struct cli_bc *bc, unsigned func, unsigned bb, unsigned char
 	    return CL_EMALFDB;
 	}
 	bcfunc->dbgnodes = cli_malloc(num*sizeof(*bcfunc->dbgnodes));
-	if (!bcfunc->dbgnodes)
+	if (!bcfunc->dbgnodes) {
+        cli_errmsg("Unable to allocate memory for dbg nodes: %s\n", num*sizeof(*bcfunc->dbgnodes));
 	    return CL_EMEM;
+    }
 	for (i=0;i<num;i++) {
 	    bcfunc->dbgnodes[i] = readNumber(buffer, &offset, len, &ok);
 	    if (!ok)
@@ -1402,7 +1421,119 @@ enum parse_state {
     PARSE_SKIP
 };
 
-int cli_bytecode_load(struct cli_bc *bc, FILE *f, struct cli_dbio *dbio, int trust)
+struct sigperf_elem {
+    const char * bc_name;
+    uint64_t usecs;
+    unsigned long run_count;
+    unsigned long match_count;
+};
+
+static int sigelem_comp(const void * a, const void * b)
+{
+    const struct sigperf_elem *ela = a;
+    const struct sigperf_elem *elb = b;
+    return elb->usecs/elb->run_count - ela->usecs/ela->run_count;
+}
+
+void cli_sigperf_print()
+{
+    struct sigperf_elem stats[MAX_BC], *elem = stats;
+    int i, elems = 0, max_name_len = 0, name_len;
+
+    memset(stats, 0, sizeof(stats));
+    for (i=0;i<MAX_BC;i++) {
+	union ev_val val;
+	uint32_t count;
+	const char * name = cli_event_get_name(g_sigevents, i*BC_EVENTS_PER_SIG);
+	cli_event_get(g_sigevents, i*BC_EVENTS_PER_SIG, &val, &count);
+	if (!count) {
+	    if (name)
+		cli_dbgmsg("No event triggered for %s\n", name);
+	    continue;
+	}
+	if (name)
+	name_len = strlen(name);
+	else
+		name_len = 0;
+	if (name_len > max_name_len)
+	    max_name_len = name_len;
+	elem->bc_name = name?name:"\"noname\"";
+	elem->usecs = val.v_int;
+	elem->run_count = count;
+	cli_event_get(g_sigevents, i*BC_EVENTS_PER_SIG+1, &val, &count);
+	elem->match_count = count;
+	elem++;
+	elems++;
+    }
+
+    cli_qsort(stats, elems, sizeof(struct sigperf_elem), sigelem_comp);
+
+    elem = stats;
+    /* name runs matches microsecs avg */
+    cli_infomsg (NULL, "%-*s %*s %*s %*s %*s\n", max_name_len, "Bytecode name",
+	    8, "#runs", 8, "#matches", 12, "usecs total", 9, "usecs avg");
+    cli_infomsg (NULL, "%-*s %*s %*s %*s %*s\n", max_name_len, "=============",
+	    8, "=====", 8, "========", 12, "===========", 9, "=========");
+    while (elem->run_count) {
+	cli_infomsg (NULL, "%-*s %*lu %*lu %*zu %*.2f\n", max_name_len, elem->bc_name,
+		     8, elem->run_count, 8, elem->match_count, 
+		12, elem->usecs, 9, (double)elem->usecs/elem->run_count);
+	elem++;
+    }
+}
+
+static void sigperf_events_init(struct cli_bc *bc)
+{
+    int ret;
+    char * bc_name;
+
+    if (!g_sigevents)
+	g_sigevents = cli_events_new(MAX_BC_SIGEVENT_ID);
+
+    if (!g_sigevents) {
+	cli_errmsg("No memory for events table\n");
+	return;
+    }
+
+    if (g_sigid > MAX_BC_SIGEVENT_ID - BC_EVENTS_PER_SIG - 1) {
+	cli_errmsg("sigperf_events_init: events table full. Increase MAX_BC\n");
+	return;
+    }
+
+    if (!(bc_name = bc->lsig)) {
+	if (!(bc_name = bc->hook_name)) {
+	    cli_dbgmsg("cli_event_define error for time event id %d\n", bc->sigtime_id);
+	    return;
+	}
+    }
+
+    cli_dbgmsg("sigperf_events_init(): adding sig ids starting %u for %s\n", g_sigid, bc_name);
+
+    /* register time event */
+    bc->sigtime_id = g_sigid;
+    ret = cli_event_define(g_sigevents, g_sigid++, bc_name, ev_time, multiple_sum);
+    if (ret) {
+	cli_errmsg("sigperf_events_init: cli_event_define() error for time event id %d\n", bc->sigtime_id);
+	bc->sigtime_id = MAX_BC_SIGEVENT_ID+1;
+	return;
+    }
+
+    /* register match count */
+    bc->sigmatch_id = g_sigid;
+    ret = cli_event_define(g_sigevents, g_sigid++, bc_name, ev_int, multiple_sum);
+    if (ret) {
+	cli_errmsg("sigperf_events_init: cli_event_define() error for matches event id %d\n", bc->sigmatch_id);
+	bc->sigmatch_id = MAX_BC_SIGEVENT_ID+1;
+	return;
+    }
+}
+
+void cli_sigperf_events_destroy()
+{
+    cli_events_free(g_sigevents);
+}
+
+int cli_bytecode_load(struct cli_bc *bc, FILE *f, struct cli_dbio *dbio, int trust, int sigperf)
 {
     unsigned row = 0, current_func = 0, bb=0;
     char *buffer;
@@ -1452,16 +1583,19 @@ int cli_bytecode_load(struct cli_bc *bc, FILE *f, struct cli_dbio *dbio, int tru
 	switch (state) {
 	    case PARSE_BC_LSIG:
 		rc = parseLSig(bc, buffer);
-		if (rc == CL_BREAK) /* skip */ {
+#if 0
+DEAD CODE
+		if (rc == CL_BREAK) /* skip */ { //FIXME: parseLSig always returns CL_SUCCESS
 		    bc->state = bc_skip;
 		    state = PARSE_SKIP;
 		    continue;
 		}
-		if (rc != CL_SUCCESS) {
+		if (rc != CL_SUCCESS) { //FIXME: parseLSig always returns CL_SUCCESS
 		    cli_errmsg("Error at bytecode line %u\n", row);
 		    free(buffer);
 		    return rc;
 		}
+#endif
 		state = PARSE_BC_TYPES;
 		break;
 	    case PARSE_BC_TYPES:
@@ -1558,6 +1692,8 @@ int cli_bytecode_load(struct cli_bc *bc, FILE *f, struct cli_dbio *dbio, int tru
     }
     free(buffer);
     cli_dbgmsg("Parsed %d functions\n", current_func);
+    if (sigperf)
+	sigperf_events_init(bc);
     if (current_func != bc->num_func && bc->state != bc_skip) {
 	cli_errmsg("Loaded less functions than declared: %u vs. %u\n",
 		   current_func, bc->num_func);
@@ -1609,13 +1745,15 @@ int cli_bytecode_run(const struct cli_all_bc *bcs, const struct cli_bc *bc, stru
 
     int test_mode = 0;
     cli_ctx *cctx =(cli_ctx*)ctx->ctx;
-    if (cctx && cctx->engine->bytecode_mode == CL_BYTECODE_MODE_TEST)
-	test_mode = 1;
 
     if (!ctx || !ctx->bc || !ctx->func)
 	return CL_ENULLARG;
     if (ctx->numParams && (!ctx->values || !ctx->operands))
 	return CL_ENULLARG;
+
+    if (cctx && cctx->engine->bytecode_mode == CL_BYTECODE_MODE_TEST)
+	test_mode = 1;
+
     if (bc->state == bc_loaded) {
 	cli_errmsg("bytecode has to be prepared either for interpreter or JIT!\n");
 	return CL_EARG;
@@ -1624,6 +1762,8 @@ int cli_bytecode_run(const struct cli_all_bc *bcs, const struct cli_bc *bc, stru
 	cli_dbgmsg("bytecode triggered but running bytecodes is disabled\n");
 	return CL_SUCCESS;
     }
+    if (cctx)
+        cli_event_time_start(cctx->perf, PERFT_BYTECODE);
     ctx->env = &bcs->env;
     context_safe(ctx);
     if (test_mode) {
@@ -1641,6 +1781,7 @@ int cli_bytecode_run(const struct cli_all_bc *bcs, const struct cli_bc *bc, stru
 	    return CL_EBYTECODE_TESTFAIL;
 	}
     }
+    cli_event_time_start(g_sigevents, bc->sigtime_id);
     if (bc->state == bc_interp || test_mode) {
 	ctx->bc_events = interp_ev;
 	memset(&func, 0, sizeof(func));
@@ -1694,6 +1835,9 @@ int cli_bytecode_run(const struct cli_all_bc *bcs, const struct cli_bc *bc, stru
 	if (ctx->outfd)
 	    cli_bcapi_extract_new(ctx, -1);
     }
+    cli_event_time_stop(g_sigevents, bc->sigtime_id);
+    if (ctx->virname)
+	cli_event_count(g_sigevents, bc->sigmatch_id);
 
     if (test_mode) {
 	unsigned interp_errors = cli_event_errors(interp_ev);
@@ -1727,7 +1871,8 @@ int cli_bytecode_run(const struct cli_all_bc *bcs, const struct cli_bc *bc, stru
 	    ok = 0;
 	}
 	/*cli_event_debug(jit_ev, BCEV_EXEC_TIME);
-	cli_event_debug(interp_ev, BCEV_EXEC_TIME);*/
+        cli_event_debug(interp_ev, BCEV_EXEC_TIME);
+	cli_event_debug(g_sigevents, bc->sigtime_id);*/
 	if (!ok) {
 	    cli_events_free(jit_ev);
 	    cli_events_free(interp_ev);
@@ -1736,6 +1881,8 @@ int cli_bytecode_run(const struct cli_all_bc *bcs, const struct cli_bc *bc, stru
     }
     cli_events_free(jit_ev);
     cli_events_free(interp_ev);
+    if (cctx)
+        cli_event_time_stop(cctx->perf, PERFT_BYTECODE);
     return ret;
 }
 
@@ -1802,6 +1949,7 @@ void cli_bytecode_destroy(struct cli_bc *bc)
     if (bc->uses_apis)
 	cli_bitset_free(bc->uses_apis);
     free(bc->lsig);
+    free(bc->hook_name);
     free(bc->globalBytes);
     memset(bc, 0, sizeof(*bc));
 }
@@ -1812,13 +1960,17 @@ void cli_bytecode_destroy(struct cli_bc *bc)
 	if (o > bc->num_globals) {\
 	    cli_errmsg("bytecode: global out of range: %u > %u, for instruction %u in function %u\n",\
 		       o, (unsigned)bc->num_globals, j, i);\
+	    free(map);\
+	    free(gmap);\
 	    return CL_EBYTECODE;\
 	}\
 	val = 0x80000000 | gmap[o];\
 	break;\
     }\
-    if (o > totValues) {\
+    if (o >= totValues) {\
 	cli_errmsg("bytecode: operand out of range: %u > %u, for instruction %u in function %u\n", o, totValues, j, i);\
+	free(map);\
+	free(gmap);\
 	return CL_EBYTECODE;\
     }\
     val = map[o]; } while (0)
@@ -1893,10 +2045,13 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
     unsigned i, j, k;
     uint64_t *gmap;
     unsigned bcglobalid = cli_apicall_maxglobal - _FIRST_GLOBAL+2;
+    int ret=CL_SUCCESS;
     bc->numGlobalBytes = 0;
     gmap = cli_malloc(bc->num_globals*sizeof(*gmap));
-    if (!gmap)
-	return CL_EMEM;
+    if (!gmap) {
+        cli_errmsg("interpreter: Unable to allocate memory for global map: %u\n", bc->num_globals*sizeof(*gmap));
+        return CL_EMEM;
+    }
     for (j=0;j<bc->num_globals;j++) {
 	uint16_t ty = bc->globaltys[j];
 	unsigned align = typealign(bc, ty);
@@ -1907,8 +2062,11 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
     }
     if (bc->numGlobalBytes) {
 	bc->globalBytes = cli_calloc(1, bc->numGlobalBytes);
-	if (!bc->globalBytes)
+	if (!bc->globalBytes) {
+        cli_errmsg("interpreter: Unable to allocate memory for globalBytes: %u\n", bc->numGlobalBytes);
+        free(gmap);
 	    return CL_EMEM;
+    }
     } else
 	bc->globalBytes = NULL;
 
@@ -1968,12 +2126,15 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
 	}
     }
 
-    for (i=0;i<bc->num_func;i++) {
+    for (i=0;i<bc->num_func && ret == CL_SUCCESS;i++) {
 	struct cli_bc_func *bcfunc = &bc->funcs[i];
 	unsigned totValues = bcfunc->numValues + bcfunc->numConstants + bc->num_globals;
 	unsigned *map = cli_malloc(sizeof(*map)*totValues);
-	if (!map)
+	if (!map) {
+        cli_errmsg("interpreter: Unable to allocate memory for map: %u\n", sizeof(*map)*totValues);
+        free(gmap);
 	    return CL_EMEM;
+    }
 	bcfunc->numBytes = 0;
 	for (j=0;j<bcfunc->numValues;j++) {
 	    uint16_t ty = bcfunc->types[j];
@@ -1992,7 +2153,7 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
 	    map[bcfunc->numValues+j] = bcfunc->numBytes;
 	    bcfunc->numBytes += 8;
 	}
-	for (j=0;j<bcfunc->numInsts;j++) {
+	for (j=0;j<bcfunc->numInsts && ret == CL_SUCCESS;j++) {
 	    struct cli_bc_inst *inst = &bcfunc->allinsts[j];
 	    inst->dest = map[inst->dest];
 	    switch (inst->opcode) {
@@ -2050,27 +2211,32 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
 			target = &bc->funcs[inst->u.ops.funcid];
 			if (inst->u.ops.funcid > bc->num_func) {
 			    cli_errmsg("bytecode: called function out of range: %u > %u\n", inst->u.ops.funcid, bc->num_func);
-			    return CL_EBYTECODE;
+			    ret = CL_EBYTECODE;
 			}
-			if (inst->u.ops.numOps != target->numArgs) {
+			else if (inst->u.ops.numOps != target->numArgs) {
 			    cli_errmsg("bytecode: call operands don't match function prototype\n");
-			    return CL_EBYTECODE;
+			    ret = CL_EBYTECODE;
 			}
 		    } else {
 			/* APIs have at most 2 parameters always */
 			if (inst->u.ops.numOps > 5) {
 			    cli_errmsg("bytecode: call operands don't match function prototype\n");
-			    return CL_EBYTECODE;
+			    ret = CL_EBYTECODE;
 			}
 		    }
-		    if (inst->u.ops.numOps) {
+		    if (ret != CL_SUCCESS)
+			break;
+		    if (inst->u.ops.numOps > 0) {
 			inst->u.ops.opsizes = cli_malloc(sizeof(*inst->u.ops.opsizes)*inst->u.ops.numOps);
 			if (!inst->u.ops.opsizes) {
 			    cli_errmsg("Out of memory when allocating operand sizes\n");
-			    return CL_EMEM;
+			    ret = CL_EMEM;
+			    break;
 			}
-		    } else
+		    } else {
 			inst->u.ops.opsizes = NULL;
+			break;
+		    }
 		    for (k=0;k<inst->u.ops.numOps;k++) {
 			MAPPTR(inst->u.ops.ops[k]);
 			if (inst->opcode == OP_BC_CALL_DIRECT)
@@ -2087,13 +2253,15 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
 		    if (inst->u.three[1]&0x80000000 ||
 			bcfunc->types[inst->u.binop[1]]&0x8000) {
                       cli_errmsg("bytecode: gep1 of alloca is not allowed\n");
-                      return CL_EBYTECODE;
+                      ret = CL_EBYTECODE;
                     }
+            if (ret != CL_SUCCESS)
+                break;
 		    MAP(inst->u.three[1]);
 		    MAP(inst->u.three[2]);
                     inst->u.three[0] = get_geptypesize(bc, inst->u.three[0]);
                     if (inst->u.three[0] == -1)
-                      return CL_EBYTECODE;
+                      ret = CL_EBYTECODE;
                     break;
 		case OP_BC_GEPZ:
 		    /*three[0] is the type*/
@@ -2104,8 +2272,9 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
 			inst->interp_op = 5*(inst->interp_op/5)+3;
 		    MAP(inst->u.three[1]);
 		    if (calc_gepz(bc, bcfunc, inst->u.three[0], inst->u.three[2]) == -1)
-			return CL_EBYTECODE;
-		    MAP(inst->u.three[2]);
+			ret = CL_EBYTECODE;
+            if (ret == CL_SUCCESS)
+		        MAP(inst->u.three[2]);
 		    break;
 /*		case OP_BC_GEPN:
 		    *TODO 
@@ -2137,14 +2306,15 @@ static int cli_bytecode_prepare_interpreter(struct cli_bc *bc)
 		    break;
 		default:
 		    cli_warnmsg("Bytecode: unhandled opcode: %d\n", inst->opcode);
-		    return CL_EBYTECODE;
+		    ret = CL_EBYTECODE;
 	    }
 	}
-	free(map);
+    if (map)
+	    free(map);
     }
     free(gmap);
     bc->state = bc_interp;
-    return CL_SUCCESS;
+    return ret;
 }
 
 static int add_selfcheck(struct cli_all_bc *bcs)
@@ -2371,7 +2541,7 @@ static int run_builtin_or_loaded(struct cli_all_bc *bcs, uint8_t kind, const cha
 	    return CL_EMALFDB;
 	}
 
-	rc = cli_bytecode_load(bc, NULL, &dbio, 1);
+	rc = cli_bytecode_load(bc, NULL, &dbio, 1, 0);
 	if (rc) {
 	    cli_errmsg("Failed to load builtin %s bytecode\n", desc);
 	    free(bc);
@@ -2640,6 +2810,9 @@ int cli_bytecode_runhook(cli_ctx *cctx, const struct cl_engine *engine, struct c
     int ret;
     unsigned executed = 0, breakflag = 0, errorflag = 0;
 
+    if (!cctx)
+        return CL_ENULLARG;
+
     cli_dbgmsg("Bytecode executing hook id %u (%u hooks)\n", id, hooks_cnt);
     /* restore match counts */
     cli_bytecode_context_setfile(ctx, map);
@@ -2662,7 +2835,7 @@ int cli_bytecode_runhook(cli_ctx *cctx, const struct cl_engine *engine, struct c
 	    continue;
 	}
 	if (ctx->virname) {
-	    cli_dbgmsg("Bytecode found virus: %s\n", ctx->virname);
+	    cli_dbgmsg("Bytecode runhook found virus: %s\n", ctx->virname);
 	    cli_append_virus(cctx, ctx->virname);
 	    if (!(cctx->options & CL_SCAN_ALLMATCHES)) {
 		cli_bytecode_context_clear(ctx);
@@ -2682,7 +2855,7 @@ int cli_bytecode_runhook(cli_ctx *cctx, const struct cl_engine *engine, struct c
 	    char *tempfile;
 	    int fd = cli_bytecode_context_getresult_file(ctx, &tempfile);
 	    if (fd && fd != -1) {
-		if (cctx && cctx->engine->keeptmp)
+		if (cctx->engine->keeptmp)
 		    cli_dbgmsg("Bytecode %u unpacked file saved in %s\n",
 			       bc->id, tempfile);
 		else
@@ -2692,11 +2865,11 @@ int cli_bytecode_runhook(cli_ctx *cctx, const struct cl_engine *engine, struct c
 		cctx->recursion++;
 		ret = cli_magic_scandesc(fd, cctx);
 		cctx->recursion--;
-		if (!cctx || !cctx->engine->keeptmp)
+		if (!cctx->engine->keeptmp)
 		    if (ftruncate(fd, 0) == -1)
 			cli_dbgmsg("ftruncate failed on %d\n", fd);
 		close(fd);
-		if (!cctx || !cctx->engine->keeptmp) {
+		if (!cctx->engine->keeptmp) {
 		    if (tempfile && cli_unlink(tempfile))
 			ret = CL_EUNLINK;
 		}
@@ -2722,7 +2895,7 @@ int cli_bytecode_runhook(cli_ctx *cctx, const struct cl_engine *engine, struct c
 	cli_dbgmsg("Bytecode: executed %u bytecodes for this hook\n", executed);
     else
 	cli_dbgmsg("Bytecode: no logical signature matched, no bytecode executed\n");
-    if (errorflag && cctx && cctx->engine->bytecode_mode == CL_BYTECODE_MODE_TEST)
+    if (errorflag && cctx->engine->bytecode_mode == CL_BYTECODE_MODE_TEST)
 	return CL_EBYTECODE_TESTFAIL;
     return breakflag ? CL_BREAK : CL_CLEAN;
 }
