@@ -161,7 +161,10 @@ uint32_t cli_bcapi_disasm_x86(struct cli_bc_ctx *ctx, struct DISASM_RESULT *res,
      * When we'll support mmx/sse instructions this should be updated! */
     n = MIN(32, ctx->fmap->len - ctx->off);
     buf = fmap_need_off_once(ctx->fmap, ctx->off, n);
-    next = cli_disasm_one(buf, n, res, 0);
+    if (buf)
+        next = cli_disasm_one(buf, n, res, 0);
+    else
+        next = NULL;
     if (!next) {
 	cli_dbgmsg("bcapi_disasm: failed\n");
 	cli_event_count(EV, BCEV_DISASM_FAIL);
@@ -499,7 +502,10 @@ int32_t cli_bcapi_extract_new(struct cli_bc_ctx *ctx, int32_t id)
     if (ctx->ctx && cli_updatelimits(ctx->ctx, ctx->written))
 	return -1;
     ctx->written = 0;
-    lseek(ctx->outfd, 0, SEEK_SET);
+    if (lseek(ctx->outfd, 0, SEEK_SET) == -1) {
+        cli_dbgmsg("bytecode: call to lseek() has failed\n");
+        return CL_ESEEK;
+    }
     cli_dbgmsg("bytecode: scanning extracted file %s\n", ctx->tempfile);
     cctx = (cli_ctx*)ctx->ctx;
     if (cctx) {
@@ -533,7 +539,7 @@ int32_t cli_bcapi_extract_new(struct cli_bc_ctx *ctx, int32_t id)
 int32_t cli_bcapi_read_number(struct cli_bc_ctx *ctx, uint32_t radix)
 {
     unsigned i;
-    char *p;
+    const char *p;
     int32_t result;
 
     if ((radix != 10 && radix != 16) || !ctx->fmap)
@@ -541,7 +547,7 @@ int32_t cli_bcapi_read_number(struct cli_bc_ctx *ctx, uint32_t radix)
     cli_event_int(EV, BCEV_OFFSET, ctx->off);
     while ((p = fmap_need_off_once(ctx->fmap, ctx->off, BUF))) {
 	for (i=0;i<BUF;i++) {
-	    if (p[i] >= '0' && p[i] <= '9') {
+	    if ((p[i] >= '0' && p[i] <= '9') || (radix == 16 && ((p[i] >= 'a' && p[i] <= 'f') || (p[i] >= 'A' && p[i] <= 'F')))) {
 		char *endptr;
 		p = fmap_need_ptr_once(ctx->fmap, p+i, 16);
 		if (!p)
@@ -607,7 +613,7 @@ int32_t cli_bcapi_hashset_contains(struct cli_bc_ctx *ctx , int32_t id, uint32_t
 int32_t cli_bcapi_hashset_empty(struct cli_bc_ctx *ctx, int32_t id)
 {
     struct cli_hashset *s = get_hashset(ctx, id);
-    return !s->count;
+    return s ? !s->count : 1;
 }
 
 int32_t cli_bcapi_hashset_done(struct cli_bc_ctx *ctx , int32_t id)
@@ -704,7 +710,7 @@ uint32_t cli_bcapi_buffer_pipe_read_avail(struct cli_bc_ctx *ctx , int32_t id)
     return ctx->file_size - b->read_cursor;
 }
 
-uint8_t* cli_bcapi_buffer_pipe_read_get(struct cli_bc_ctx *ctx , int32_t id, uint32_t size)
+const uint8_t* cli_bcapi_buffer_pipe_read_get(struct cli_bc_ctx *ctx , int32_t id, uint32_t size)
 {
     struct bc_buffer *b = get_buffer(ctx, id);
     if (!b || size > cli_bcapi_buffer_pipe_read_avail(ctx, id) || !size)
@@ -842,7 +848,7 @@ int32_t cli_bcapi_inflate_process(struct cli_bc_ctx *ctx , int32_t id)
     b->stream.avail_in = avail_in_orig =
 	cli_bcapi_buffer_pipe_read_avail(ctx, b->from);
 
-    b->stream.next_in = cli_bcapi_buffer_pipe_read_get(ctx, b->from,
+    b->stream.next_in = (void*)cli_bcapi_buffer_pipe_read_get(ctx, b->from,
 						       b->stream.avail_in);
 
     b->stream.avail_out = avail_out_orig =
@@ -958,7 +964,7 @@ static struct bc_jsnorm *get_jsnorm(struct cli_bc_ctx *ctx, int32_t id)
 int32_t cli_bcapi_jsnorm_process(struct cli_bc_ctx *ctx, int32_t id)
 {
     unsigned avail;
-    unsigned char *in;
+    const unsigned char *in;
     cli_ctx *cctx = ctx->ctx;
     struct bc_jsnorm *b = get_jsnorm(ctx, id);
     if (!b || b->from == -1 || !b->state)
@@ -1447,7 +1453,7 @@ uint32_t cli_bcapi_pdf_getobjsize(struct cli_bc_ctx *ctx , int32_t objidx)
     return ctx->pdf_objs[objidx+1].start - ctx->pdf_objs[objidx].start - 4;
 }
 
-uint8_t* cli_bcapi_pdf_getobj(struct cli_bc_ctx *ctx , int32_t objidx, uint32_t amount)
+const uint8_t* cli_bcapi_pdf_getobj(struct cli_bc_ctx *ctx , int32_t objidx, uint32_t amount)
 {
     uint32_t size = cli_bcapi_pdf_getobjsize(ctx, objidx);
     if (amount > size)
