@@ -150,8 +150,10 @@ static always_inline void* cli_stack_alloc(struct stack *stack, unsigned bytes)
     }
     /* not enough room here, allocate new chunk */
     chunk = cli_malloc(sizeof(*stack->chunk));
-    if (!chunk)
-	return NULL;
+    if (!chunk) {
+        cli_warnmsg("cli_stack_alloc: Unable to allocate memory for stack-chunk: bytes: %u!\n", sizeof(*stack->chunk));
+        return NULL;
+    }
 
     *(uint16_t*)&chunk->u.data[last_size_off] = stack->last_size;
     stack->last_size = bytes/sizeof(align_t);
@@ -599,7 +601,7 @@ static inline void* ptr_torealptr(const struct ptr_infos *infos, int64_t ptr,
 
 static always_inline int check_sdivops(int64_t op0, int64_t op1)
 {
-    return op1 == 0 || (op0 == -1 && op1 ==  (-9223372036854775807LL-1LL));
+    return op1 == 0 || (op1 == -1 && op0 ==  (-9223372036854775807LL-1LL));
 }
 
 static unsigned globaltypesize(uint16_t id)
