@@ -25,6 +25,8 @@
 #include "others.h"
 
 void cache_add(unsigned char *md5, size_t size, cli_ctx *ctx);
+/* Removes a hash from the cache */
+void cache_remove(unsigned char *md5, size_t size, const struct cl_engine *engine);
 int cache_check(unsigned char *hash, cli_ctx *ctx);
 int cli_cache_init(struct cl_engine *engine);
 void cli_cache_destroy(struct cl_engine *engine);
