@@ -105,12 +105,20 @@ typedef struct lzx_content_tag {
 /* Read in a block of data from either the mmap area or the given fd */
 static int chm_read_data(fmap_t *map, char *dest, off_t offset, off_t len)
 {
-    void *src = fmap_need_off_once(map, offset, len);
+    const void *src = fmap_need_off_once(map, offset, len);
     if(!src) return FALSE;
     memcpy(dest, src, len);
     return TRUE;
 }
 
+/* Read callback for lzx compressed data */
+static int chm_readn(struct cab_file *file, unsigned char *buffer, int bytes) {
+    int ret = fmap_readn(file->cab->map, buffer, file->cab->cur_offset, bytes);
+    if(ret > 0)
+	file->cab->cur_offset += ret;
+    return ret;
+}
+
 static uint64_t chm_copy_file_data(int ifd, int ofd, uint64_t len)
 {
 	unsigned char data[8192];
@@ -225,10 +233,10 @@ static int itsp_read_header(chm_metadata_t *metadata, off_t offset)
 	return TRUE;
 }
 
-static uint64_t read_enc_int(char **start, char *end)
+static uint64_t read_enc_int(const char **start, const char *end)
 {
 	uint64_t retval=0;
-	char *current;
+	const char *current;
 	
 	current = *start;
 	
@@ -250,7 +258,7 @@ static uint64_t read_enc_int(char **start, char *end)
 /* Read control entries */
 static int read_control_entries(chm_metadata_t *metadata)
 {
-	char *name;
+	const char *name;
 	uint64_t name_len, section, offset, length;
 
 	while (metadata->chunk_entries--) {
@@ -486,7 +494,7 @@ static int read_sys_reset_table(chm_metadata_t *metadata, lzx_reset_table_t *lzx
 /* This section interfaces to the mspack files. As such, this is a */
 /* little bit dirty compared to my usual code */
 
-static int chm_decompress_stream(int fd, chm_metadata_t *metadata, const char *dirname, cli_ctx *ctx)
+static int chm_decompress_stream(chm_metadata_t *metadata, const char *dirname, cli_ctx *ctx)
 {
 	lzx_content_t lzx_content;
 	lzx_reset_table_t lzx_reset_table;
@@ -495,6 +503,7 @@ static int chm_decompress_stream(int fd, chm_metadata_t *metadata, const char *d
 	struct lzx_stream * stream;
 	char filename[1024];
 	struct cab_file file;
+	struct cab_archive cab;
 	
 	snprintf(filename, 1024, "%s"PATHSEP"clamav-unchm.bin", dirname);
 	tmpfd = open(filename, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRWXU);
@@ -555,15 +564,15 @@ static int chm_decompress_stream(int fd, chm_metadata_t *metadata, const char *d
 	length &= -lzx_control.reset_interval;
 	
 	cli_dbgmsg("Compressed offset: %lu\n", (unsigned long int) lzx_content.offset);
-	if ((uint64_t) lseek(fd, lzx_content.offset, SEEK_SET) != lzx_content.offset) {
-		goto abort;
-	}
 
 	memset(&file, 0, sizeof(struct cab_file));
 	file.max_size = ctx->engine->maxfilesize;
-	stream = lzx_init(fd, tmpfd, window_bits,
+	file.cab = &cab;
+	cab.map = metadata->map;
+	cab.cur_offset = lzx_content.offset;
+	stream = lzx_init(tmpfd, window_bits,
 			lzx_control.reset_interval / LZX_FRAME_SIZE,
-			4096, length, &file, NULL);
+			4096, length, &file, chm_readn);
 	if (!stream) {
 		cli_dbgmsg("lzx_init failed\n");
 		goto abort;
@@ -608,7 +617,6 @@ void cli_chm_close(chm_metadata_t *metadata)
 	if (metadata->ufd >= 0) {
 		close(metadata->ufd);
 	}
-	funmap(metadata->map);
 }
 
 int cli_chm_extract_file(char *dirname, chm_metadata_t *metadata, cli_ctx *ctx)
@@ -659,9 +667,9 @@ int cli_chm_prepare_file(chm_metadata_t *metadata)
 	return retval;
 }
 
-int cli_chm_open(int fd, const char *dirname, chm_metadata_t *metadata, cli_ctx *ctx)
+int cli_chm_open(const char *dirname, chm_metadata_t *metadata, cli_ctx *ctx)
 {
-	struct stat statbuf;
+	STATBUF statbuf;
 	int retval;
 
 	cli_dbgmsg("in cli_chm_open\n");
@@ -670,20 +678,10 @@ int cli_chm_open(int fd, const char *dirname, chm_metadata_t *metadata, cli_ctx
 		return retval;
 	}
 
-	if (fstat(fd, &statbuf) == 0) {
-		if (statbuf.st_size < CHM_ITSF_MIN_LEN) {
-			return CL_ESTAT;
-		}
-		metadata->m_length = statbuf.st_size;
-		metadata->map = fmap(fd, 0, metadata->m_length);
-		if (!metadata->map) {
-			return CL_EMAP;
-		}
-	} else {
-	    char err[128];
-	    cli_warnmsg("fstat() failed: %s\n", cli_strerror(errno, err, sizeof(err)));
-	    return CL_ESTAT;
-	}
+	metadata->map = *ctx->fmap;
+	if(metadata->map->len < CHM_ITSF_MIN_LEN)
+		return CL_ESTAT;
+	metadata->m_length = metadata->map->len;
 
 	if (!itsf_read_header(metadata)) {
 		goto abort;
@@ -730,7 +728,7 @@ int cli_chm_open(int fd, const char *dirname, chm_metadata_t *metadata, cli_ctx
 		goto abort;
 	}
 	
-	metadata->ufd = chm_decompress_stream(fd, metadata, dirname, ctx);
+	metadata->ufd = chm_decompress_stream(metadata, dirname, ctx);
 	if (metadata->ufd == -1) {
 		goto abort;
 	}
@@ -743,6 +741,5 @@ int cli_chm_open(int fd, const char *dirname, chm_metadata_t *metadata, cli_ctx
 	return CL_SUCCESS;
 
 abort:
-	funmap(metadata->map);
 	return CL_EFORMAT;
 }
