@@ -108,14 +108,14 @@ typedef struct chm_metadata_tag {
 	int ofd;
 	uint32_t num_chunks;
 	off_t chunk_offset;
-	char *chunk_data;
-	char *chunk_current;
-	char *chunk_end;
+	const char *chunk_data;
+	const char *chunk_current;
+	const char *chunk_end;
 	fmap_t *map;
 	uint16_t chunk_entries;
 } chm_metadata_t;
 
-int cli_chm_open(int fd, const char *dirname, chm_metadata_t *metadata, cli_ctx *ctx);
+int cli_chm_open(const char *dirname, chm_metadata_t *metadata, cli_ctx *ctx);
 int cli_chm_prepare_file(chm_metadata_t *metadata);
 int cli_chm_extract_file(char *dirname, chm_metadata_t *metadata, cli_ctx *ctx);
 void cli_chm_close(chm_metadata_t *metadata);
