@@ -46,15 +46,26 @@
 
 #define TAR_BLOCKSIZE 512
 
+static void cli_untgz_cleanup(char *path, gzFile infile, FILE *outfile, int fdd)
+{
+    cli_dbgmsg("in cli_untgz_cleanup()\n");
+    if (path != NULL)
+        free (path);
+    if (infile != NULL) 
+        gzclose (infile);
+    if (outfile != NULL)
+        fclose(outfile);
+}
+
 static int cli_untgz(int fd, const char *destdir)
 {
 	char *path, osize[13], name[101], type;
 	char block[TAR_BLOCKSIZE];
-	int nbytes, nread, nwritten, in_block = 0, fdd;
+	int nbytes, nread, nwritten, in_block = 0, fdd = -1;
 	unsigned int size, pathlen = strlen(destdir) + 100 + 5;
 	FILE *outfile = NULL;
-	struct stat foo;
-	gzFile infile;
+	STATBUF foo;
+	gzFile infile = NULL;
 
 
     cli_dbgmsg("in cli_untgz()\n");
@@ -66,7 +77,7 @@ static int cli_untgz(int fd, const char *destdir)
 
     if((infile = gzdopen(fdd, "rb")) == NULL) {
 	cli_errmsg("cli_untgz: Can't gzdopen() descriptor %d, errno = %d\n", fdd, errno);
-	if(fstat(fdd, &foo) == 0)
+	if(FSTAT(fdd, &foo) == 0)
 	    close(fdd);
 	return -1;
     }
@@ -74,7 +85,7 @@ static int cli_untgz(int fd, const char *destdir)
     path = (char *) cli_calloc(sizeof(char), pathlen);
     if(!path) {
 	cli_errmsg("cli_untgz: Can't allocate memory for path\n");
-	gzclose(infile);
+	cli_untgz_cleanup(NULL, infile, NULL, fdd);
 	return -1;
     }
 
@@ -87,8 +98,7 @@ static int cli_untgz(int fd, const char *destdir)
 
 	if(nread != TAR_BLOCKSIZE) {
 	    cli_errmsg("cli_untgz: Incomplete block read\n");
-	    free(path);
-	    gzclose(infile);
+	    cli_untgz_cleanup(path, infile, outfile, fdd);
 	    return -1;
 	}
 
@@ -101,8 +111,7 @@ static int cli_untgz(int fd, const char *destdir)
 
 	    if(strchr(name, '/')) {
 		cli_errmsg("cli_untgz: Slash separators are not allowed in CVD\n");
-		free(path);
-	        gzclose(infile);
+		cli_untgz_cleanup(path, infile, outfile, fdd);
 		return -1;
 	    }
 
@@ -116,13 +125,11 @@ static int cli_untgz(int fd, const char *destdir)
 		    break;
 		case '5':
 		    cli_errmsg("cli_untgz: Directories are not supported in CVD\n");
-		    free(path);
-	            gzclose(infile);
+		    cli_untgz_cleanup(path, infile, outfile, fdd);
 		    return -1;
 		default:
 		    cli_errmsg("cli_untgz: Unknown type flag '%c'\n", type);
-		    free(path);
-	            gzclose(infile);
+		    cli_untgz_cleanup(path, infile, outfile, fdd);
 		    return -1;
 	    }
 	    in_block = 1;
@@ -130,8 +137,8 @@ static int cli_untgz(int fd, const char *destdir)
 	    if(outfile) {
 		if(fclose(outfile)) {
 		    cli_errmsg("cli_untgz: Cannot close file %s\n", path);
-		    free(path);
-	            gzclose(infile);
+		    outfile = NULL;
+		    cli_untgz_cleanup(path, infile, outfile, fdd);
 		    return -1;
 		}
 		outfile = NULL;
@@ -139,8 +146,7 @@ static int cli_untgz(int fd, const char *destdir)
 
 	    if(!(outfile = fopen(path, "wb"))) {
 		cli_errmsg("cli_untgz: Cannot create file %s\n", path);
-		free(path);
-	        gzclose(infile);
+		cli_untgz_cleanup(path, infile, outfile, fdd);
 		return -1;
 	    }
 
@@ -149,9 +155,7 @@ static int cli_untgz(int fd, const char *destdir)
 
 	    if((sscanf(osize, "%o", &size)) == 0) {
 		cli_errmsg("cli_untgz: Invalid size in header\n");
-		free(path);
-	        gzclose(infile);
-		fclose(outfile);
+		cli_untgz_cleanup(path, infile, outfile, fdd);
 		return -1;
 	    }
 
@@ -161,8 +165,7 @@ static int cli_untgz(int fd, const char *destdir)
 
 	    if(nwritten != nbytes) {
 		cli_errmsg("cli_untgz: Wrote %d instead of %d (%s)\n", nwritten, nbytes, path);
-		free(path);
-	        gzclose(infile);
+		cli_untgz_cleanup(path, infile, outfile, fdd);
 		return -1;
 	    }
 
@@ -172,14 +175,27 @@ static int cli_untgz(int fd, const char *destdir)
 	}
     }
 
-    if(outfile)
-	fclose(outfile);
-
-    gzclose(infile);
-    free(path);
+    cli_untgz_cleanup(path, infile, outfile, fdd);
     return 0;
 }
 
+static void cli_tgzload_cleanup(int comp, struct cli_dbio *dbio, int fdd)
+{
+    cli_dbgmsg("in cli_tgzload_cleanup()\n");
+    if(comp) {
+        gzclose(dbio->gzs);
+        dbio->gzs = NULL;
+    }
+    else {
+        fclose(dbio->fs);
+        dbio->fs = NULL;
+    }
+    if(dbio->buf != NULL) {
+        free(dbio->buf);
+        dbio->buf = NULL;
+    }
+}
+
 static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, unsigned int options, struct cli_dbio *dbio, struct cli_dbinfo *dbinfo)
 {
 	char osize[13], name[101];
@@ -190,22 +206,21 @@ static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, un
 	struct cli_dbinfo *db;
 	unsigned char hash[32];
 
-#define CLOSE_DBIO	    \
-    if(compr)		    \
-	gzclose(dbio->gzs);  \
-    else		    \
-	fclose(dbio->fs)
-
     cli_dbgmsg("in cli_tgzload()\n");
 
-    lseek(fd, 512, SEEK_SET);
+    if(lseek(fd, 512, SEEK_SET) < 0) {
+        return CL_ESEEK;
+    }
+
     if(cli_readn(fd, block, 7) != 7)
 	return CL_EFORMAT; /* truncated file? */
 
     if(!strncmp(block, "COPYING", 7))
 	compr = 0;
 
-    lseek(fd, 512, SEEK_SET);
+    if(lseek(fd, 512, SEEK_SET) < 0) {
+        return CL_ESEEK;
+    }
 
     if((fdd = dup(fd)) == -1) {
 	cli_errmsg("cli_tgzload: Can't duplicate descriptor %d\n", fd);
@@ -215,12 +230,16 @@ static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, un
     if(compr) {
 	if((dbio->gzs = gzdopen(fdd, "rb")) == NULL) {
 	    cli_errmsg("cli_tgzload: Can't gzdopen() descriptor %d, errno = %d\n", fdd, errno);
+	    if (fdd > -1)
+		close(fdd);
 	    return CL_EOPEN;
 	}
 	dbio->fs = NULL;
     } else {
 	if((dbio->fs = fdopen(fdd, "rb")) == NULL) {
 	    cli_errmsg("cli_tgzload: Can't fdopen() descriptor %d, errno = %d\n", fdd, errno);
+	    if (fdd > -1)
+		close(fdd);
 	    return CL_EOPEN;
 	}
 	dbio->gzs = NULL;
@@ -230,7 +249,7 @@ static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, un
     dbio->buf = cli_malloc(dbio->bufsize);
     if(!dbio->buf) {
 	cli_errmsg("cli_tgzload: Can't allocate memory for dbio->buf\n");
-	CLOSE_DBIO;
+	cli_tgzload_cleanup(compr, dbio, fdd);
 	return CL_EMALFDB;
     }
     dbio->bufpt = NULL;
@@ -249,8 +268,7 @@ static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, un
 
 	if(nread != TAR_BLOCKSIZE) {
 	    cli_errmsg("cli_tgzload: Incomplete block read\n");
-	    free(dbio->buf);
-	    CLOSE_DBIO;
+	    cli_tgzload_cleanup(compr, dbio, fdd);
 	    return CL_EMALFDB;
 	}
 
@@ -262,8 +280,7 @@ static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, un
 
 	if(strchr(name, '/')) {
 	    cli_errmsg("cli_tgzload: Slash separators are not allowed in CVD\n");
-	    free(dbio->buf);
-	    CLOSE_DBIO;
+	    cli_tgzload_cleanup(compr, dbio, fdd);
 	    return CL_EMALFDB;
 	}
 
@@ -275,13 +292,11 @@ static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, un
 		break;
 	    case '5':
 		cli_errmsg("cli_tgzload: Directories are not supported in CVD\n");
-		free(dbio->buf);
-		CLOSE_DBIO;
+		cli_tgzload_cleanup(compr, dbio, fdd);
 		return CL_EMALFDB;
 	    default:
 		cli_errmsg("cli_tgzload: Unknown type flag '%c'\n", type);
-		free(dbio->buf);
-		CLOSE_DBIO;
+		cli_tgzload_cleanup(compr, dbio, fdd);
 		return CL_EMALFDB;
 	}
 
@@ -290,8 +305,7 @@ static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, un
 
 	if((sscanf(osize, "%o", &size)) == 0) {
 	    cli_errmsg("cli_tgzload: Invalid size in header\n");
-	    free(dbio->buf);
-	    CLOSE_DBIO;
+	    cli_tgzload_cleanup(compr, dbio, fdd);
 	    return CL_EMALFDB;
 	}
 	dbio->size = size;
@@ -311,13 +325,11 @@ static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, un
 	    ret = cli_load(name, engine, signo, options, dbio);
 	    if(ret) {
 		cli_errmsg("cli_tgzload: Can't load %s\n", name);
-		free(dbio->buf);
-		CLOSE_DBIO;
+		cli_tgzload_cleanup(compr, dbio, fdd);
 		return CL_EMALFDB;
 	    }
 	    if(!dbinfo) {
-		free(dbio->buf);
-		CLOSE_DBIO;
+		cli_tgzload_cleanup(compr, dbio, fdd);
 		return CL_SUCCESS;
 	    } else {
 		db = dbinfo;
@@ -325,22 +337,19 @@ static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, un
 		    db = db->next;
 		if(!db) {
 		    cli_errmsg("cli_tgzload: File %s not found in .info\n", name);
-		    free(dbio->buf);
-		    CLOSE_DBIO;
+		    cli_tgzload_cleanup(compr, dbio, fdd);
 		    return CL_EMALFDB;
 		}
 		if(dbio->bread) {
 		    if(db->size != dbio->bread) {
 			cli_errmsg("cli_tgzload: File %s not correctly loaded\n", name);
-			free(dbio->buf);
-			CLOSE_DBIO;
+			cli_tgzload_cleanup(compr, dbio, fdd);
 			return CL_EMALFDB;
 		    }
 		    sha256_final(&dbio->sha256ctx, hash);
 		    if(memcmp(db->hash, hash, 32)) {
 			cli_errmsg("cli_tgzload: Invalid checksum for file %s\n", name);
-			free(dbio->buf);
-			CLOSE_DBIO;
+			cli_tgzload_cleanup(compr, dbio, fdd);
 			return CL_EMALFDB;
 		    }
 		}
@@ -360,8 +369,7 @@ static int cli_tgzload(int fd, struct cl_engine *engine, unsigned int *signo, un
 	}
     }
 
-    free(dbio->buf);
-    CLOSE_DBIO;
+    cli_tgzload_cleanup(compr, dbio, fdd);
     return CL_SUCCESS;
 }
 
@@ -488,7 +496,7 @@ void cl_cvdfree(struct cl_cvd *cvd)
     free(cvd);
 }
 
-static int cli_cvdverify(FILE *fs, struct cl_cvd *cvdpt, unsigned int cld)
+static int cli_cvdverify(FILE *fs, struct cl_cvd *cvdpt, unsigned int skipsig)
 {
 	struct cl_cvd *cvd;
 	char *md5, head[513];
@@ -510,7 +518,7 @@ static int cli_cvdverify(FILE *fs, struct cl_cvd *cvdpt, unsigned int cld)
     if(cvdpt)
 	memcpy(cvdpt, cvd, sizeof(struct cl_cvd));
 
-    if(cld) {
+    if(skipsig) {
 	cl_cvdfree(cvd);
 	return CL_SUCCESS;
     }
@@ -567,7 +575,7 @@ int cl_cvdverify(const char *file)
     return ret;
 }
 
-int cli_cvdload(FILE *fs, struct cl_engine *engine, unsigned int *signo, unsigned int options, unsigned int cld, const char *filename, unsigned int chkonly)
+int cli_cvdload(FILE *fs, struct cl_engine *engine, unsigned int *signo, unsigned int options, unsigned int dbtype, const char *filename, unsigned int chkonly)
 {
 	struct cl_cvd cvd, dupcvd;
 	FILE *dupfs;
@@ -581,32 +589,34 @@ int cli_cvdload(FILE *fs, struct cl_engine *engine, unsigned int *signo, unsigne
     cli_dbgmsg("in cli_cvdload()\n");
 
     /* verify */
-    if((ret = cli_cvdverify(fs, &cvd, cld)))
+    if((ret = cli_cvdverify(fs, &cvd, dbtype)))
 	return ret;
 
-    /* check for duplicate db */
-    dupname = cli_strdup(filename);
-    if(!dupname)
-	return CL_EMEM;
-    dupname[strlen(dupname) - 2] = (cld ? 'v' : 'l');
-    if(!access(dupname, R_OK) && (dupfs = fopen(dupname, "rb"))) {
-	if((ret = cli_cvdverify(dupfs, &dupcvd, !cld))) {
+    if(dbtype <= 1) {
+	/* check for duplicate db */
+	dupname = cli_strdup(filename);
+	if(!dupname)
+	    return CL_EMEM;
+	dupname[strlen(dupname) - 2] = (dbtype == 1 ? 'v' : 'l');
+	if(!access(dupname, R_OK) && (dupfs = fopen(dupname, "rb"))) {
+	    if((ret = cli_cvdverify(dupfs, &dupcvd, !dbtype))) {
+		fclose(dupfs);
+		free(dupname);
+		return ret;
+	    }
 	    fclose(dupfs);
-	    free(dupname);
-	    return ret;
-	}
-	fclose(dupfs);
-	if(dupcvd.version > cvd.version) {
-	    cli_warnmsg("Detected duplicate databases %s and %s. The %s database is older and will not be loaded, you should manually remove it from the database directory.\n", filename, dupname, filename);
-	    free(dupname);
-	    return CL_SUCCESS;
-	} else if(dupcvd.version == cvd.version && !cld) {
-	    cli_warnmsg("Detected duplicate databases %s and %s, please manually remove one of them\n", filename, dupname);
-	    free(dupname);
-	    return CL_SUCCESS;
+	    if(dupcvd.version > cvd.version) {
+		cli_warnmsg("Detected duplicate databases %s and %s. The %s database is older and will not be loaded, you should manually remove it from the database directory.\n", filename, dupname, filename);
+		free(dupname);
+		return CL_SUCCESS;
+	    } else if(dupcvd.version == cvd.version && !dbtype) {
+		cli_warnmsg("Detected duplicate databases %s and %s, please manually remove one of them\n", filename, dupname);
+		free(dupname);
+		return CL_SUCCESS;
+	    }
 	}
+	free(dupname);
     }
-    free(dupname);
 
     if(strstr(filename, "daily.")) {
 	time(&s_time);
@@ -636,7 +646,10 @@ int cli_cvdload(FILE *fs, struct cl_engine *engine, unsigned int *signo, unsigne
 
     cfd = fileno(fs);
     dbio.chkonly = 0;
-    ret = cli_tgzload(cfd, engine, signo, options | CL_DB_OFFICIAL, &dbio, NULL);
+    if(dbtype == 2)
+	ret = cli_tgzload(cfd, engine, signo, options | CL_DB_UNSIGNED, &dbio, NULL);
+    else
+	ret = cli_tgzload(cfd, engine, signo, options | CL_DB_OFFICIAL, &dbio, NULL);
     if(ret != CL_SUCCESS)
 	return ret;
 
@@ -646,12 +659,18 @@ int cli_cvdload(FILE *fs, struct cl_engine *engine, unsigned int *signo, unsigne
 	return CL_EMALFDB;
     }
     dbinfo = engine->dbinfo ? engine->dbinfo->next : NULL;
-    if(!dbinfo)
+    if(!dbinfo) {
+	cli_errmsg("cli_cvdload: dbinfo error\n");
 	return CL_EMALFDB;
+    }
 
     dbio.chkonly = chkonly;
-    options |= CL_DB_SIGNED;
-    ret = cli_tgzload(cfd, engine, signo, options | CL_DB_OFFICIAL, &dbio, dbinfo);
+    if(dbtype == 2)
+	options |= CL_DB_UNSIGNED;
+    else
+	options |= CL_DB_SIGNED | CL_DB_OFFICIAL;
+
+    ret = cli_tgzload(cfd, engine, signo, options, &dbio, dbinfo);
 
     while(engine->dbinfo) {
 	dbinfo = engine->dbinfo;
