@@ -37,4 +37,10 @@
 #define CLI_DEFAULT_MIN_CC_COUNT    3
 #define CLI_DEFAULT_MIN_SSN_COUNT   3
 
+#define CLI_DEFAULT_MAXEMBEDDEDPE       10485760
+#define CLI_DEFAULT_MAXHTMLNORMALIZE    10485760
+#define CLI_DEFAULT_MAXHTMLNOTAGS       2097152
+#define CLI_DEFAULT_MAXSCRIPTNORMALIZE  5242880
+#define CLI_DEFAULT_MAXZIPTYPERCG       1048576
+
 #endif
