@@ -1,5 +1,7 @@
 /*
- *  Copyright (C) 2004 aCaB <acab@clamav.net>
+ *  Copyright (C) 2007-2008 Sourcefire, Inc.
+ *
+ *  Authors: Alberto Wu
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -22,8 +24,8 @@
 #include "cltypes.h"
 #include "execs.h"
 
-int unfsg_200(char *, char *, int, int, uint32_t, uint32_t, uint32_t, int);
-int unfsg_133(char *, char *, int , int, struct cli_exe_section *, int, uint32_t, uint32_t, int);
+int unfsg_200(const char *, char *, int, int, uint32_t, uint32_t, uint32_t, int);
+int unfsg_133(const char *, char *, int , int, struct cli_exe_section *, int, uint32_t, uint32_t, int);
 
 #endif
 
