@@ -1,7 +1,7 @@
 /*
  *  Hash-table and -set data structures.
  *
- *  Copyright (C) 2007-2008 Sourcefire, Inc.
+ *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
  *
@@ -326,11 +326,18 @@ const struct cli_htu32_element *cli_htu32_next(const struct cli_htu32 *s, const
 static int cli_hashtab_grow(struct cli_hashtable *s)
 {
 	const size_t new_capacity = nearest_power(s->capacity + 1);
-	struct cli_element* htable = cli_calloc(new_capacity, sizeof(*s->htable));
+	struct cli_element* htable;
 	size_t i,idx, used = 0;
+
 	cli_dbgmsg("hashtab.c: new capacity: %lu\n",new_capacity);
-	if(new_capacity == s->capacity || !htable)
+	if(new_capacity == s->capacity) {
+		cli_errmsg("hashtab.c: capacity problem growing from: %lu\n",s->capacity);
 		return CL_EMEM;
+	}
+	htable = cli_calloc(new_capacity, sizeof(*s->htable));
+	if(!htable) {
+		return CL_EMEM;
+	}
 
 	PROFILE_GROW_START(s);
 	cli_dbgmsg("hashtab.c: Warning: growing open-addressing hashtables is slow. Either allocate more storage when initializing, or use other hashtable types!\n");
@@ -355,6 +362,7 @@ static int cli_hashtab_grow(struct cli_hashtable *s)
 			}
 			else {
 				cli_errmsg("hashtab.c: Impossible - unable to rehash table");
+				free (htable);
 				return CL_EMEM;/* this means we didn't find enough room for all elements in the new table, should never happen */ 
 			}
 		}
@@ -450,8 +458,10 @@ const struct cli_element* cli_hashtab_insert(struct cli_hashtable *s, const char
 					PROFILE_INSERT(s, tries);
 				}
 				thekey = cli_malloc(len+1);
-				if(!thekey)
+				if(!thekey) {
+                    cli_errmsg("hashtab.c: Unable to allocate memory for thekey\n");
 					return NULL;
+                }
 				strncpy(thekey, key, len+1);
 				thekey[len]='\0';
 				element->key = thekey;
@@ -661,11 +671,13 @@ int cli_hashset_init(struct cli_hashset* hs, size_t initial_capacity, uint8_t lo
 	hs->keys = cli_malloc(initial_capacity * sizeof(*hs->keys));
 	hs->mempool = NULL;
 	if(!hs->keys) {
+        cli_errmsg("hashtab.c: Uable to allocate memory for hs->keys\n");
 		return CL_EMEM;
 	}
 	hs->bitmap = cli_calloc(initial_capacity >> 5, sizeof(*hs->bitmap));
 	if(!hs->bitmap) {
 		free(hs->keys);
+        cli_errmsg("hashtab.c: Unable to allocate memory for hs->bitmap\n");
 		return CL_EMEM;
 	}
 	return 0;
@@ -684,11 +696,13 @@ int cli_hashset_init_pool(struct cli_hashset* hs, size_t initial_capacity, uint8
 	hs->mempool = mempool;
 	hs->keys = mpool_malloc(mempool, initial_capacity * sizeof(*hs->keys));
 	if(!hs->keys) {
+        cli_errmsg("hashtab.c: Unable to allocate memory pool for hs->keys\n");
 		return CL_EMEM;
 	}
 	hs->bitmap = mpool_calloc(mempool, initial_capacity >> 5, sizeof(*hs->bitmap));
 	if(!hs->bitmap) {
 		mpool_free(mempool, hs->keys);
+        cli_errmsg("hashtab.c: Unable to allocate/initialize memory for hs->keys\n");
 		return CL_EMEM;
 	}
 	return 0;
@@ -819,6 +833,7 @@ ssize_t cli_hashset_toarray(const struct cli_hashset* hs, uint32_t** array)
 	}
 	*array = arr = cli_malloc(hs->count * sizeof(*arr));
 	if(!arr) {
+        cli_errmsg("hashtab.c: Unable to allocate memory for array\n");
 		return CL_EMEM;
 	}
 
@@ -926,8 +941,10 @@ int  cli_map_setvalue(struct cli_map *m, const void* value, int32_t valuesize)
 	if (v->value)
 	    free(v->value);
 	v->value = cli_malloc(valuesize);
-	if (!v->value)
-	    return -CL_EMEM;
+	if (!v->value) {
+        cli_errmsg("hashtab.c: Unable to allocate  memory for v->value\n");
+        return -CL_EMEM;
+    }
 	memcpy(v->value, value, valuesize);
 	v->valuesize = valuesize;
     }
