@@ -86,6 +86,39 @@ void cli_hashtab_free(struct cli_hashtable *s);
 int cli_hashtab_load(FILE* in, struct cli_hashtable *s);
 int cli_hashtab_store(const struct cli_hashtable *s,FILE* out);
 
+
+struct cli_htu32_element {
+    uint32_t key;
+    union {
+	unsigned long as_ulong;
+	void *as_ptr;
+    } data;
+};
+
+struct cli_htu32 {
+    struct cli_htu32_element* htable;
+    size_t capacity;
+    size_t used;
+    size_t maxfill;/* 80% */
+
+    STRUCT_PROFILE
+};
+
+#ifndef USE_MPOOL
+#define cli_htu32_init(A, B, C) cli_htu32_init(A, B)
+#define cli_htu32_insert(A, B, C) cli_htu32_insert(A, B)
+#define cli_htu32_free(A, B) cli_htu32_free(A)
+#endif
+int cli_htu32_init(struct cli_htu32 *s, size_t capacity, mpool_t *mempool);
+int cli_htu32_insert(struct cli_htu32 *s, const struct cli_htu32_element *item, mpool_t *mempool);
+const struct cli_htu32_element *cli_htu32_find(const struct cli_htu32 *s, uint32_t key);
+void cli_htu32_delete(struct cli_htu32 *s, uint32_t key);
+void cli_htu32_clear(struct cli_htu32 *s);
+void cli_htu32_free(struct cli_htu32 *s, mpool_t *mempool);
+const struct cli_htu32_element *cli_htu32_next(const struct cli_htu32 *s, const struct cli_htu32_element *current);
+size_t cli_htu32_numitems(struct cli_htu32 *s);
+
+
 /* a hashtable that stores the values too */
 struct cli_map_value {
     void *value;
