@@ -2,7 +2,7 @@
  *  Normalise HTML text.
  *  Decode MS Script Encoder protection. 
  *
- *  Copyright (C) 2007-2008 Sourcefire, Inc.
+ *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Trog
  *
@@ -174,6 +174,7 @@ static unsigned char *cli_readchunk(FILE *stream, m_area_t *m_area, unsigned int
 
 	chunk = (unsigned char *) cli_malloc(max_len);
 	if (!chunk) {
+        cli_errmsg("readchunk: Unable to allocate memory for chunk\n");
 		return NULL;
 	}
 
@@ -192,7 +193,8 @@ static unsigned char *cli_readchunk(FILE *stream, m_area_t *m_area, unsigned int
 		    ptr = m_area->buffer + m_area->offset;
 		start = ptr;
 		end = ptr - m_area->offset + m_area->length;
-		if (start >= end) {
+
+		if ((start >= end) || !start) {
 			free(chunk);
 			return NULL;
 		}
@@ -218,6 +220,10 @@ static unsigned char *cli_readchunk(FILE *stream, m_area_t *m_area, unsigned int
 			}
 			if(m_area->map)
 			    ptr = (unsigned char *)fmap_need_ptr_once(m_area->map, ptr, end - ptr);
+			if (!ptr) {
+			    cli_warnmsg("fmap inconsistency\n");
+			    ptr = end;
+			}
 			/* we have unknown number of NULL chars,
 			 * copy char-by-char and skip them */
 			while((ptr < end) && (chunk_len < max_len-1)) {
@@ -515,8 +521,10 @@ static inline void html_tag_contents_done(tag_arguments_t *tags,int idx, struct
 	unsigned char *p;
 	cont->contents[cont->pos++] = '\0';
 	p = cli_malloc(cont->pos);
-	if(!p)
+	if(!p) {
+        cli_errmsg("html_tag_contents_done: Unable to allocate memory for p\n");
 		return;
+    }
 	memcpy(p, cont->contents, cont->pos);
 	tags->contents[idx-1] = p;
 	cont->pos = 0;
@@ -641,7 +649,7 @@ static int cli_html_normalise(int fd, m_area_t *m_area, const char *dirname, tag
 	html_state state=HTML_NORM, next_state=HTML_BAD_STATE, saved_next_state=HTML_BAD_STATE;
 	char filename[1024], tag[HTML_STR_LENGTH+1], tag_arg[HTML_STR_LENGTH+1];
 	char tag_val[HTML_STR_LENGTH+1], *tmp_file, *arg_value;
-	unsigned char *line, *ptr, *ptr_screnc = NULL;
+	unsigned char *line = NULL, *ptr, *ptr_screnc = NULL;
 	tag_arguments_t tag_args;
 	quoted_state quoted = NOT_QUOTED;
 	unsigned long length = 0;
@@ -697,6 +705,7 @@ static int cli_html_normalise(int fd, m_area_t *m_area, const char *dirname, tag
 
 		file_buff_o2 = (file_buff_t *) cli_malloc(sizeof(file_buff_t));
 		if (!file_buff_o2) {
+            cli_errmsg("cli_html_normalise: Unable to allocate memory for file_buff_o2\n");
 			file_buff_o2 = file_buff_text = NULL;
 			goto abort;
 		}
@@ -716,6 +725,7 @@ static int cli_html_normalise(int fd, m_area_t *m_area, const char *dirname, tag
 			close(file_buff_o2->fd);
 			free(file_buff_o2);
 			file_buff_o2 = file_buff_text = NULL;
+            cli_errmsg("cli_html_normalise: Unable to allocate memory for file_buff_text\n");
 			goto abort;
 		}
 
@@ -1591,6 +1601,7 @@ static int cli_html_normalise(int fd, m_area_t *m_area, const char *dirname, tag
 				if (dirname) {
 					file_tmp_o1 = (file_buff_t *) cli_malloc(sizeof(file_buff_t));
 					if (!file_tmp_o1) {
+                        cli_errmsg("cli_html_normalise: Unable to allocate memory for file_tmp_o1\n");
 						goto abort;
 					}
 					snprintf(filename, 1024, "%s"PATHSEP"rfc2397", dirname);
@@ -1770,6 +1781,8 @@ static int cli_html_normalise(int fd, m_area_t *m_area, const char *dirname, tag
 	}
 	retval = TRUE;
 abort:
+	if (line) /* only needed for abort case */
+		free(line);
 	if (in_form_action)
 		free(in_form_action);
         if (in_ahref) /* tag not closed, force closing */
@@ -1830,36 +1843,29 @@ int html_normalise_map(fmap_t *map, const char *dirname, tag_arguments_t *hrefs,
 	return retval;
 }
 
-int html_screnc_decode(int fd, const char *dirname)
+int html_screnc_decode(fmap_t *map, const char *dirname)
 {
-	int fd_tmp, count, retval=FALSE;
+	int count, retval=FALSE;
 	unsigned char *line, tmpstr[6];
 	unsigned char *ptr, filename[1024];
-	FILE *stream_in;
 	int ofd;
 	struct screnc_state screnc_state;
+	m_area_t m_area;
 
-	lseek(fd, 0, SEEK_SET);
-	fd_tmp = dup(fd);
-	if (fd_tmp < 0) {
-		return FALSE;
-	}
-	stream_in = fdopen(fd_tmp, "r");
-	if (!stream_in) {
-		close(fd_tmp);
-		return FALSE;
-	}
+	memset(&m_area, 0, sizeof(m_area));
+	m_area.length = map->len;
+	m_area.offset = 0;
+	m_area.map = map;
 
 	snprintf((char*)filename, 1024, "%s"PATHSEP"screnc.html", dirname);
 	ofd = open((const char*)filename, O_WRONLY|O_CREAT|O_TRUNC, S_IWUSR|S_IRUSR);
 
 	if (ofd < 0) {
 		cli_dbgmsg("open failed: %s\n", filename);
-		fclose(stream_in);
 		return FALSE;
 	}
 
-	while ((line = cli_readchunk(stream_in, NULL, 8192)) != NULL) {
+	while ((line = cli_readchunk(NULL, &m_area, 8192)) != NULL) {
 		ptr = (unsigned char*)strstr((char*)line, "#@~^");
 		if (ptr) {
 			break;
@@ -1876,7 +1882,7 @@ int html_screnc_decode(int fd, const char *dirname)
 	do {
 		if (! *ptr) {
 			free(line);
-			ptr = line = cli_readchunk(stream_in, NULL, 8192);
+			ptr = line = cli_readchunk(NULL, &m_area, 8192);
 			if (!line) {
 				goto abort;
 			}
@@ -1903,7 +1909,7 @@ int html_screnc_decode(int fd, const char *dirname)
 		cli_writen(ofd, ptr, strlen((const char*)ptr));
 		free(line);
 		if (screnc_state.length) {
-			ptr = line = cli_readchunk(stream_in, NULL, 8192);
+			ptr = line = cli_readchunk(NULL, &m_area, 8192);
 		}
 	}
 	cli_writen(ofd, "</script>",strlen("</script>"));
@@ -1912,7 +1918,6 @@ int html_screnc_decode(int fd, const char *dirname)
 	retval = TRUE;
 
 abort:
-	fclose(stream_in);
 	close(ofd);
 	return retval;
 }
