@@ -193,7 +193,7 @@ static const uint8_t skey[] = { 0xec, 0xca, 0x79, 0xf8 }; /* ~0x13, ~0x35, ~0x86
 
 /* Extracts the content of MSI based IS */
 int cli_scanishield_msi(cli_ctx *ctx, off_t off) {
-    uint8_t *buf;
+    const uint8_t *buf;
     unsigned int fcount, scanned = 0;
     int ret;
     fmap_t *map = *ctx->fmap;
@@ -298,7 +298,10 @@ int cli_scanishield_msi(cli_ctx *ctx, off_t off) {
 	if (ret == CL_SUCCESS) {
 	    cli_dbgmsg("ishield-msi: extracted to %s\n", tempfile);
 
-	    lseek(ofd, 0, SEEK_SET);
+	    if (lseek(ofd, 0, SEEK_SET) == -1) {
+            cli_dbgmsg("ishield-msi: call to lseek() failed\n");
+            ret = CL_ESEEK;
+        }
 	    ret = cli_magic_scandesc(ofd, ctx);
 	}
 	close(ofd);
@@ -337,12 +340,14 @@ static int is_extract_cab(cli_ctx *ctx, uint64_t off, uint64_t size, uint64_t cs
 
 /* Extract the content of older (non-MSI) IS */
 int cli_scanishield(cli_ctx *ctx, off_t off, size_t sz) {
-    char *fname, *path, *version, *strsz, *eostr, *data;
+    const char *fname, *path, *version, *strsz, *data;
+    char *eostr;
     int ret = CL_CLEAN;
     long fsize;
     off_t coff = off;
     struct IS_CABSTUFF c = { NULL, -1, 0, 0 };
     fmap_t *map = *ctx->fmap;
+    unsigned fc = 0;
 
     while(ret == CL_CLEAN) {
 	fname = fmap_need_offstr(map, coff, 2048);
@@ -371,6 +376,10 @@ int cli_scanishield(cli_ctx *ctx, off_t off, size_t sz) {
 	) break;
 
 	cli_dbgmsg("ishield: @%lx found file %s (%s) - version %s - size %lu\n", (unsigned long int) coff, fname, path, version, (unsigned long int) fsize);
+	if(cli_matchmeta(ctx, fname, fsize, fsize, 0, fc++, 0, NULL) == CL_VIRUS) {
+	    ret = CL_VIRUS;
+	    break;
+	}
 	sz -= (data - fname) + fsize;
 
 	if(!strncasecmp(fname, "data", 4)) {
@@ -431,7 +440,8 @@ int cli_scanishield(cli_ctx *ctx, off_t off, size_t sz) {
 
 /* Utility func to scan a fd @ a given offset and size */
 static int is_dump_and_scan(cli_ctx *ctx, off_t off, size_t fsize) {
-    char *fname, *buf;
+    char *fname;
+    const char *buf;
     int ofd, ret = CL_CLEAN;
     fmap_t *map = *ctx->fmap;
 
@@ -463,7 +473,10 @@ static int is_dump_and_scan(cli_ctx *ctx, off_t off, size_t fsize) {
     }
     if(!fsize) {
 	cli_dbgmsg("ishield: extracted to %s\n", fname);
-	lseek(ofd, 0, SEEK_SET);
+	if (lseek(ofd, 0, SEEK_SET) == -1) {
+        cli_dbgmsg("ishield: call to lseek() failed\n");
+        ret = CL_ESEEK;
+    }
 	ret = cli_magic_scandesc(ofd, ctx);
     }
     close(ofd);
@@ -481,7 +494,7 @@ static int is_parse_hdr(cli_ctx *ctx, struct IS_CABSTUFF *c) {
     char hash[33], *hdr;
     fmap_t *map = *ctx->fmap;
 
-    struct IS_HDR *h1;
+    const struct IS_HDR *h1;
     struct IS_OBJECTS *objs;
     /* struct IS_INSTTYPEHDR *typehdr; -- UNUSED */
 
@@ -670,7 +683,8 @@ static void md5str(uint8_t *sum) {
 #define IS_CABBUFSZ 65536
 
 static int is_extract_cab(cli_ctx *ctx, uint64_t off, uint64_t size, uint64_t csize) {
-    uint8_t *inbuf, *outbuf;
+    const uint8_t *inbuf;
+    uint8_t *outbuf;
     char *tempfile;
     int ofd, ret = CL_CLEAN;
     z_stream z;
@@ -678,8 +692,10 @@ static int is_extract_cab(cli_ctx *ctx, uint64_t off, uint64_t size, uint64_t cs
     int success = 0;
     fmap_t *map = *ctx->fmap;
 
-    if(!(outbuf = cli_malloc(IS_CABBUFSZ)))
-	return CL_EMEM;
+    if(!(outbuf = cli_malloc(IS_CABBUFSZ))) {
+        cli_errmsg("is_extract_cab: Unable to allocate memory for outbuf\n");
+        return CL_EMEM;
+    }
 
     if(!(tempfile = cli_gentemp(ctx->engine->tmpdir))) {
 	free(outbuf);
@@ -758,7 +774,8 @@ static int is_extract_cab(cli_ctx *ctx, uint64_t off, uint64_t size, uint64_t cs
 	    cli_dbgmsg("is_extract_cab: extracted %llu bytes to %s, expected %llu, scanning anyway.\n", (long long)outsz, tempfile, (long long)size);
 	else
 	    cli_dbgmsg("is_extract_cab: extracted to %s\n", tempfile);
-	lseek(ofd, 0, SEEK_SET);
+	if (lseek(ofd, 0, SEEK_SET) == -1)
+        cli_dbgmsg("is_extract_cab: call to lseek() failed\n");
 	ret = cli_magic_scandesc(ofd, ctx);
     }
 
