@@ -35,7 +35,7 @@
 #include <assert.h>
 
 #include "cltypes.h"
-#include "lexglobal.h"
+#include "jsparse/lexglobal.h"
 #include "hashtab.h"
 #include "others.h"
 #include "str.h"
@@ -498,11 +498,6 @@ static int match_parameters(const yystype *tokens, const char ** param_names, si
 static const char *de_packer_3[] = {"p","a","c","k","e","r"};
 static const char *de_packer_2[] = {"p","a","c","k","e","d"};
 
-
-#ifndef MAX
-#define MAX(a, b) ((a)>(b) ? (a) : (b))
-#endif
-
 static inline char *textbuffer_done(yyscan_t scanner)
 {
        char *str = cli_realloc(scanner->buf.data, scanner->buf.pos);
@@ -904,7 +899,7 @@ void cli_js_output(struct parser_state *state, const char *tempdir)
 	char lastchar = '\0';
 	char filename[1024];
 
-	snprintf(filename, 1024, "%s/javascript", tempdir);
+	snprintf(filename, 1024, "%s"PATHSEP"javascript", tempdir);
 
 	buf.pos = 0;
 	buf.outfd = open(filename, O_CREAT | O_WRONLY, 0600);
@@ -1404,10 +1399,6 @@ static inline int parseId(YYSTYPE *lvalp, yyscan_t scanner)
 	return 0;
 }
 
-#ifndef MIN
-#define MIN(a,b) ((a)<(b) ? (a):(b))
-#endif
-
 static int parseOperator(YYSTYPE *lvalp, yyscan_t scanner)
 {
 	size_t len = MIN(5, scanner->insize - scanner->pos);
