@@ -349,7 +349,17 @@ static int ac_maketrans(struct cli_matcher *root)
 		struct cli_ac_node *failtarget = node->fail;
 		while(IS_LEAF(failtarget) || !failtarget->trans[i])
 		    failtarget = failtarget->fail;
-		node->trans[i] = failtarget->trans[i];
+		failtarget = failtarget->trans[i];
+		node->trans[i] = failtarget;
+	    } else if (IS_FINAL(child) && IS_LEAF(child)) {
+		struct cli_ac_patt *list;
+		list = child->list;
+		if (list) {
+		    while (list->next) list = list->next;
+		    list->next = child->fail->list;
+		} else
+		    child->list = child->fail->list;
+		child->trans = child->fail->trans;
 	    } else {
 		if((ret = bfs_enqueue(&bfs, &bfs_last, child)) != 0)
 		    return ret;
@@ -459,9 +469,16 @@ void cli_ac_free(struct cli_matcher *root)
     if(root->ac_reloff)
 	mpool_free(root->mempool, root->ac_reloff);
 
+    /* Freeing trans nodes must be done before freeing table nodes! */
     for(i = 0; i < root->ac_nodes; i++) {
-	if(!IS_LEAF(root->ac_nodetable[i]))
+	if(!IS_LEAF(root->ac_nodetable[i]) &&
+	   root->ac_nodetable[i]->fail &&
+	   root->ac_nodetable[i]->trans != root->ac_nodetable[i]->fail->trans) {
 	    mpool_free(root->mempool, root->ac_nodetable[i]->trans);
+	}
+    }
+
+    for(i = 0; i < root->ac_nodes; i++) {
 	mpool_free(root->mempool, root->ac_nodetable[i]);
     }
 
@@ -1033,6 +1050,7 @@ void cli_ac_freedata(struct cli_ac_data *data)
 	    }
 	}
 	free(data->offmatrix);
+	data->offmatrix = NULL;
 	data->partsigs = 0;
     }
 
@@ -1052,6 +1070,7 @@ void cli_ac_freedata(struct cli_ac_data *data)
     }
 }
 
+/* returns only CL_SUCCESS or CL_EMEM */
 inline static int ac_addtype(struct cli_matched_type **list, cli_file_t type, off_t offset, const cli_ctx *ctx)
 {
 	struct cli_matched_type *tnode, *tnode_last;
@@ -1145,14 +1164,14 @@ void cli_ac_chkmacro(struct cli_matcher *root, struct cli_ac_data *data, unsigne
 }
 
 
-int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **virname, void **customdata, struct cli_ac_result **res, const struct cli_matcher *root, struct cli_ac_data *mdata, uint32_t offset, cli_file_t ftype, struct cli_matched_type **ftoffset, unsigned int mode, const cli_ctx *ctx)
+int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **virname, void **customdata, struct cli_ac_result **res, const struct cli_matcher *root, struct cli_ac_data *mdata, uint32_t offset, cli_file_t ftype, struct cli_matched_type **ftoffset, unsigned int mode, cli_ctx *ctx)
 {
 	struct cli_ac_node *current;
-	struct cli_ac_patt *patt, *pt, *faillist;
+	struct cli_ac_patt *patt, *pt;
         uint32_t i, bp, realoff, matchend;
 	uint16_t j;
+	uint8_t found, viruses_found = 0;
 	int32_t **offmatrix, swp;
-	uint8_t found;
 	int type = CL_CLEAN;
 	struct cli_ac_result *newres;
 
@@ -1169,21 +1188,13 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
     for(i = 0; i < length; i++)  {
 	current = current->trans[buffer[i]];
 
-	if(IS_FINAL(current)) {
-	    faillist = NULL;
+	if(UNLIKELY(IS_FINAL(current))) {
+	    struct cli_ac_patt *faillist = current->fail->list;
 	    patt = current->list;
-	    if(IS_LEAF(current)) {
-		current = current->fail;
-		faillist = current->list;
-	    }
-	    while(1) {
-		if(!patt) {
-		    if(!(patt = faillist))
-			break;
-		    faillist = NULL;
-		}
+	    while(patt) {
 		if(patt->partno > mdata->min_partno) {
-		    patt = NULL;
+		    patt = faillist;
+		    faillist = NULL;
 		    continue;
 		}
 		bp = i + 1 - patt->depth;
@@ -1244,6 +1255,7 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 			}
 			if(pt->sigid) { /* it's a partial signature */
 
+			    /* if 2nd or later part, confirm some prior part has matched */
 			    if(pt->partno != 1 && (!mdata->offmatrix[pt->sigid - 1] || !mdata->offmatrix[pt->sigid - 1][pt->partno - 2][0])) {
 				pt = pt->next_same;
 				continue;
@@ -1252,6 +1264,7 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 			    if(pt->partno + 1 > mdata->min_partno)
 				mdata->min_partno = pt->partno + 1;
 
+			    /* sparsely populated matrix, so allocate and initialize if NULL */
 			    if(!mdata->offmatrix[pt->sigid - 1]) {
 				mdata->offmatrix[pt->sigid - 1] = cli_malloc(pt->parts * sizeof(int32_t *));
 				if(!mdata->offmatrix[pt->sigid - 1]) {
@@ -1306,7 +1319,7 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 
 			    if(pt->partno == 1 || (found && (pt->partno != pt->parts))) {
 				if(offmatrix[pt->partno - 1][0] == CLI_DEFAULT_AC_TRACKLEN + 1)
-				    offmatrix[pt->partno - 1][0] = 1;
+				    offmatrix[pt->partno - 1][0] = 1; /* wrap, ends up at 2 */
 				offmatrix[pt->partno - 1][0]++;
 				offmatrix[pt->partno - 1][offmatrix[pt->partno - 1][0]] = offset + matchend;
 
@@ -1344,8 +1357,10 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 
 				    if(res) {
 					newres = (struct cli_ac_result *) malloc(sizeof(struct cli_ac_result));
-					if(!newres)
+					if(!newres) {
+                        cli_errmsg("cli_ac_scanbuff: Can't allocate memory for newres %u\n", sizeof(struct cli_ac_result));
 					    return CL_EMEM;
+                    }
 					newres->virname = pt->virname;
 					newres->customdata = pt->customdata;
 					newres->next = *res;
@@ -1355,11 +1370,18 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 					pt = pt->next_same;
 					continue;
 				    } else {
-					if(virname)
+					if(ctx && SCAN_ALL) {
+					    cli_append_virus(ctx, (const char *)pt->virname);
+					    viruses_found = 1;
+					}
+					if (virname)
 					    *virname = pt->virname;
 					if(customdata)
 					    *customdata = pt->customdata;
-					return CL_VIRUS;
+					if (!ctx || !SCAN_ALL)
+					    return CL_VIRUS;
+					pt = pt->next_same;
+					continue;
 				    }
 				}
 			    }
@@ -1370,6 +1392,7 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 				    return CL_TYPE_IGNORED;
 
 				if((pt->type > type || pt->type >= CL_TYPE_SFX || pt->type == CL_TYPE_MSEXE) && (!pt->rtype || ftype == pt->rtype)) {
+
 				    cli_dbgmsg("Matched signature for file type %s at %u\n", pt->virname, realoff);
 				    type = pt->type;
 				    if(ftoffset && (!*ftoffset || (*ftoffset)->cnt < MAX_EMBEDDED_OBJ || type == CL_TYPE_ZIPSFX) && (type >= CL_TYPE_SFX || ((ftype == CL_TYPE_MSEXE || ftype == CL_TYPE_ZIP || ftype == CL_TYPE_MSOLE2) && type == CL_TYPE_MSEXE)))  {
@@ -1387,8 +1410,10 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 
 				if(res) {
 				    newres = (struct cli_ac_result *) malloc(sizeof(struct cli_ac_result));
-				    if(!newres)
-					return CL_EMEM;
+				    if(!newres) {
+                        cli_errmsg("cli_ac_scanbuff: Can't allocate memory for newres %u\n", sizeof(struct cli_ac_result));
+                        return CL_EMEM;
+                    }
 				    newres->virname = pt->virname;
 				    newres->customdata = pt->customdata;
 				    newres->offset = realoff;
@@ -1398,11 +1423,18 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 				    pt = pt->next_same;
 				    continue;
 				} else {
-				    if(virname)
+				    if(ctx && SCAN_ALL) {
+					cli_append_virus(ctx, (const char *)pt->virname);
+					viruses_found = 1;
+				    }
+				    if (virname)
 					*virname = pt->virname;
 				    if(customdata)
 					*customdata = pt->customdata;
-				    return CL_VIRUS;
+				    if (!ctx || !SCAN_ALL)
+					return CL_VIRUS;
+				    pt = pt->next_same;
+				    continue;
 				}
 			    }
 			}
@@ -1414,6 +1446,8 @@ int cli_ac_scanbuff(const unsigned char *buffer, uint32_t length, const char **v
 	}
     }
 
+    if (viruses_found)
+	return CL_VIRUS;
     return (mode & AC_SCAN_FT) ? type : CL_CLEAN;
 }
 
@@ -1641,16 +1675,11 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 		newspecial->type = AC_SPECIAL_WHITE;
 	    */
 	    } else {
+		newspecial->num = 1;
 		for(i = 0; i < strlen(pt); i++)
 		    if(pt[i] == '|')
 			newspecial->num++;
 
-		if(!newspecial->num) {
-		    error = CL_EMALFDB;
-		    break;
-		} else
-		    newspecial->num++;
-
 		if(3 * newspecial->num - 1 == (uint16_t) strlen(pt)) {
 		    newspecial->type = AC_SPECIAL_ALT_CHAR;
 		    newspecial->str = (unsigned char *) mpool_malloc(root->mempool, newspecial->num);
@@ -1664,13 +1693,21 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 		}
 
 		for(i = 0; i < newspecial->num; i++) {
-		    if(!(h = cli_strtok(pt, i, "|"))) {
-			error = CL_EMALFDB;
-			break;
-		    }
+			unsigned int clen;
 
-		    if(!(c = (char*)cli_mpool_hex2str(root->mempool, h))) {
+		    if(newspecial->num == 1) {
+			c = (char *) cli_mpool_hex2str(root->mempool, pt);
+			clen = strlen(pt) / 2;
+		    } else {
+			if(!(h = cli_strtok(pt, i, "|"))) {
+			    error = CL_EMEM;
+			    break;
+			}
+			c = (char *) cli_mpool_hex2str(root->mempool, h);
+			clen = strlen(h) / 2;
 			free(h);
+		    }
+		    if(!c) {
 			error = CL_EMALFDB;
 			break;
 		    }
@@ -1689,19 +1726,17 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 				cli_errmsg("cli_ac_addsig: Can't allocate specialpt->next\n");
 				error = CL_EMEM;
 				free(c);
-				free(h);
 				break;
 			    }
 			    specialpt->next->str = (unsigned char *) c;
-			    specialpt->next->len = strlen(h) / 2;
+			    specialpt->next->len = clen;
 			} else {
 			    newspecial->str = (unsigned char *) c;
-			    newspecial->len = strlen(h) / 2;
+			    newspecial->len = clen;
 			}
 		    }
-		    free(h);
 		}
-		if(newspecial->type == AC_SPECIAL_ALT_CHAR)
+		if(newspecial->num > 1 && newspecial->type == AC_SPECIAL_ALT_CHAR)
 		    cli_qsort(newspecial->str, newspecial->num, sizeof(unsigned char), qcompare);
 
 		if(error)
@@ -1716,8 +1751,8 @@ int cli_ac_addsig(struct cli_matcher *root, const char *virname, const char *hex
 	free(hexcpy);
 
 	if(error) {
+	    free(hex);
 	    if(new->special) {
-		free(hex);
 		mpool_ac_free_special(root->mempool, new);
 	    }
 	    mpool_free(root->mempool, new);
