@@ -93,6 +93,7 @@ struct cli_matcher {
 
     /* HASH */
     struct cli_hash_patt hm;
+    struct cli_hash_wild hwild;
 
     /* Extended Aho-Corasick */
     uint32_t ac_partsigs, ac_nodes, ac_patterns, ac_lsigs;
@@ -137,7 +138,7 @@ struct cli_mtarget {
     uint8_t enable_prefiltering;
 };
 
-#define CLI_MTARGETS 10
+#define CLI_MTARGETS 13
 static const struct cli_mtarget cli_mtargets[CLI_MTARGETS] =  {
     { 0,                    "GENERIC",      0,  0, 1 },
     { CL_TYPE_MSEXE,        "PE",           1,  0, 1 },
@@ -148,7 +149,10 @@ static const struct cli_mtarget cli_mtargets[CLI_MTARGETS] =  {
     { CL_TYPE_ELF,          "ELF",          6,  1, 0 },
     { CL_TYPE_TEXT_ASCII,   "ASCII",        7,  1, 1 },
     { CL_TYPE_ERROR,        "NOT USED",     8,  1, 0 },
-    { CL_TYPE_MACHO,        "MACH-O",       9,  1, 0 }
+    { CL_TYPE_MACHO,        "MACH-O",       9,  1, 0 },
+    { CL_TYPE_PDF,          "PDF",         10,  1, 0 },
+    { CL_TYPE_SWF,          "FLASH",       11,  1, 0 },
+    { CL_TYPE_JAVA,         "JAVA",        12,  1, 0 }
 };
 
 #define CLI_OFF_ANY         0xffffffff
@@ -161,6 +165,7 @@ static const struct cli_mtarget cli_mtargets[CLI_MTARGETS] =  {
 #define CLI_OFF_SX_PLUS     6
 #define CLI_OFF_VERSION     7
 #define CLI_OFF_MACRO       8
+#define CLI_OFF_SE	    9
 
 int cli_scanbuff(const unsigned char *buffer, uint32_t length, uint32_t offset, cli_ctx *ctx, cli_file_t ftype, struct cli_ac_data **acdata);
 
