@@ -426,6 +426,11 @@ messageAddArgument(message *m, const char *arg)
 	}
 
 	p = m->mimeArguments[offset] = rfc2231(arg);
+	if(!p) {
+		/* problem inside rfc2231() */
+		cli_dbgmsg("messageAddArgument, error from rfc2231()\n");
+		return;
+	}
 
 	if(strchr(p, '=') == NULL) {
 		if(strncmp(p, "filename", 8) == 0) {
@@ -436,7 +441,7 @@ messageAddArgument(message *m, const char *arg)
 			cli_dbgmsg("Possible data corruption fixed\n");
 			p[8] = '=';
 		} else {
-			if(p && *p)
+			if(*p)
 				cli_dbgmsg("messageAddArgument, '%s' contains no '='\n", p);
 			free(m->mimeArguments[offset]);
 			m->mimeArguments[offset] = NULL;
@@ -450,7 +455,7 @@ messageAddArgument(message *m, const char *arg)
 	 * mime. By pretending defaulting to an application rather than
 	 * to nomime we can ensure they're saved and scanned
 	 */
-	if(p && ((strncasecmp(p, "filename=", 9) == 0) || (strncasecmp(p, "name=", 5) == 0)))
+	if((strncasecmp(p, "filename=", 9) == 0) || (strncasecmp(p, "name=", 5) == 0))
 		if(messageGetMimeType(m) == NOMIME) {
 			cli_dbgmsg("Force mime encoding to application\n");
 			messageSetMimeType(m, "application");
@@ -539,8 +544,15 @@ messageAddArguments(message *m, const char *s)
 				return;
 
 			ptr = strchr(kcopy, '=');
-			if(ptr == NULL)
+			if(ptr == NULL) {
 				ptr = strchr(kcopy, ':');
+                if (ptr == NULL) {
+                    cli_dbgmsg("Can't parse header \"%s\"\n", s);
+                    free(kcopy);
+                    return;
+                }
+            }
+
 			*ptr = '\0';
 
 			string = strchr(++cptr, '"');
@@ -877,8 +889,10 @@ messageAddLine(message *m, line_t *line)
 		m->body_last = m->body_last->t_next;
 	}
 
-	if(m->body_last == NULL)
+	if(m->body_last == NULL) {
+        cli_errmsg("messageAddLine: out of memory for m->body_last\n");
 		return -1;
+    }
 
 	m->body_last->t_next = NULL;
 
@@ -1075,7 +1089,7 @@ messageMoveText(message *m, text *t, message *old_message)
 		} else {
 			m->body_last = m->body_first = textMove(NULL, t);
 			if(m->body_first == NULL)
-				rc = -1;
+				return -1;
 			else
 				rc = 0;
 		}
@@ -2281,8 +2295,10 @@ rfc2231(const char *in)
 
 		/* Don't handle continuations, decode what we can */
 		p = ret = cli_malloc(strlen(in) + 16);
-		if(ret == NULL)
+		if(ret == NULL) {
+            cli_errmsg("rfc2331: out of memory, unable to proceed\n");
 			return NULL;
+        }
 
 		do {
 			switch(*in) {
@@ -2335,8 +2351,10 @@ rfc2231(const char *in)
 
 	ret = cli_malloc(strlen(in) + 1);
 
-	if(ret == NULL)
+	if(ret == NULL) {
+        cli_errmsg("rfc2331: out of memory for ret\n");
 		return NULL;
+    }
 
 	/*
 	 * memcpy(out, in, (ptr - in));
@@ -2567,8 +2585,10 @@ push(LINK1 *top, const char *string)
 
 	if((element = (LINK1)cli_malloc(sizeof(ELEMENT1))) == NULL)
 		return OUT_OF_MEMORY;
-	if((element->d1 = cli_strdup(string)) == NULL)
+	if((element->d1 = cli_strdup(string)) == NULL) {
+		free (element);
 		return OUT_OF_MEMORY;
+	}
 	element->next = *top;
 	*top = element;
 
