@@ -28,11 +28,11 @@
 #include "cltypes.h"
 
 struct lzmastate {
-	char *p0;
+	const char *p0;
 	uint32_t p1, p2;
 };
 
-int mew_lzma(char *, char *, uint32_t, uint32_t, uint32_t);
+int mew_lzma(char *, const char *, uint32_t, uint32_t, uint32_t);
 
 uint32_t lzma_upack_esi_00(struct lzmastate *, char *, char *, uint32_t);
 uint32_t lzma_upack_esi_50(struct lzmastate *, uint32_t, uint32_t, char **, char *, uint32_t *, char *, uint32_t);
