@@ -1,6 +1,6 @@
 /*
  * This file includes code from libmspack adapted for libclamav by
- * tkojm@clamav.net
+ * tkojm@clamav.net and draynor@sourcefire.com
  *
  * Copyright (C) 2003-2004 Stuart Caie
  *
@@ -124,14 +124,24 @@ static const unsigned short mszip_bit_mask_tab[17] = {
 } while (0)
 
 static int mszip_read_input(struct mszip_stream *zip) {
-  int nread = zip->read_cb ? zip->read_cb(zip->file, zip->inbuf, (int)zip->inbuf_size) : cli_readn(zip->fd, zip->inbuf, (int)zip->inbuf_size);
+  int nread = zip->read_cb(zip->file, zip->inbuf, (int)zip->inbuf_size);
   if (nread < 0) {
-    if (zip->file->error == CL_BREAK)
-      return zip->error = CL_BREAK;
+    if (zip->file->error == CL_BREAK) {
+      if (nread == zip->last) {
+        cli_dbgmsg("mszip_read_input: Two consecutive CL_BREAKs reached.\n");
+        return CL_BREAK;
+      }
+      // Need short circuit to ensure scanning small files
+      cli_dbgmsg("mszip_read_input: First CL_BREAK reached.\n");
+      zip->i_ptr = zip->i_end;
+      zip->last = nread;
+      return CL_SUCCESS;
+    }
     else
       return zip->error = CL_EFORMAT;
   }
 
+  zip->last = nread;
   zip->i_ptr = &zip->inbuf[0];
   zip->i_end = &zip->inbuf[nread];
 
@@ -489,12 +499,12 @@ static int mszip_inflate(struct mszip_stream *zip) {
 	}
 	else {
 	  code -= 257;
-	  if (code > 29) return INF_ERR_LITCODE;
+	  if (code >= 29) return INF_ERR_LITCODE;
 	  MSZIP_READ_BITS_T(length, mszip_lit_extrabits[code]);
 	  length += mszip_lit_lengths[code];
 
 	  MSZIP_READ_HUFFSYM(DISTANCE, code);
-	  if (code > 30) return INF_ERR_DISTCODE;
+	  if (code >= 30) return INF_ERR_DISTCODE;
 	  MSZIP_READ_BITS_T(distance, mszip_dist_extrabits[code]);
 	  distance += mszip_dist_offsets[code];
 
@@ -580,8 +590,7 @@ static int mszip_flush_window(struct mszip_stream *zip,
   return 0;
 }
 
-struct mszip_stream *mszip_init(int fd,
-				  int ofd,
+struct mszip_stream *mszip_init(int ofd,
 				  int input_buffer_size,
 				  int repair_mode,
 				  struct cab_file *file,
@@ -594,18 +603,19 @@ struct mszip_stream *mszip_init(int fd,
 
   /* allocate decompression state */
   if (!(zip = cli_calloc(1, sizeof(struct mszip_stream)))) {
+      cli_errmsg("mszip_stream: Unable to allocate zip buffer\n");
     return NULL;
   }
 
   /* allocate input buffer */
   zip->inbuf  = cli_malloc((size_t) input_buffer_size);
   if (!zip->inbuf) {
+      cli_errmsg("mszip_stream: Unable to allocate input buffer\n");
     free(zip);
     return NULL;
   }
 
   /* initialise decompression state */
-  zip->fd	       = fd;
   zip->ofd	       = ofd;
   zip->wflag	       = 1;
   zip->inbuf_size      = input_buffer_size;
@@ -624,7 +634,7 @@ struct mszip_stream *mszip_init(int fd,
   return zip;
 }
 
-int mszip_decompress(struct mszip_stream *zip, off_t out_bytes) {
+int mszip_decompress(struct mszip_stream *zip, uint32_t out_bytes) {
   /* for the bit buffer */
   register unsigned int bit_buffer;
   register int bits_left;
@@ -633,7 +643,7 @@ int mszip_decompress(struct mszip_stream *zip, off_t out_bytes) {
   int i, ret, state, error;
 
   /* easy answers */
-  if (!zip || (out_bytes < 0)) return CL_ENULLARG;
+  if (!zip) return CL_ENULLARG;
   if (zip->error) return zip->error;
 
   /* flush out any stored-up bytes before we begin */
@@ -769,7 +779,7 @@ void mszip_free(struct mszip_stream *zip) {
 } while (0)
 
 static int lzx_read_input(struct lzx_stream *lzx) {
-  int bread = lzx->read_cb ? lzx->read_cb(lzx->file, &lzx->inbuf[0], (int)lzx->inbuf_size) : cli_readn(lzx->fd, &lzx->inbuf[0], (int)lzx->inbuf_size);
+  int bread = lzx->read_cb(lzx->file, &lzx->inbuf[0], (int)lzx->inbuf_size);
   if (bread < 0) {
     if (lzx->file->error == CL_BREAK)
       return lzx->error = CL_BREAK;
@@ -1004,8 +1014,7 @@ static void lzx_reset_state(struct lzx_stream *lzx) {
 
 /*-------- main LZX code --------*/
 
-struct lzx_stream *lzx_init(int fd,
-			      int ofd,
+struct lzx_stream *lzx_init(int ofd,
 			      int window_bits,
 			      int reset_interval,
 			      int input_buffer_size,
@@ -1055,7 +1064,6 @@ struct lzx_stream *lzx_init(int fd,
   }
 
   /* initialise decompression state */
-  lzx->fd              = fd;
   lzx->ofd	       = ofd;
   lzx->wflag	       = 1;
   lzx->offset          = 0;
@@ -1093,7 +1101,7 @@ void lzx_set_output_length(struct lzx_stream *lzx, off_t out_bytes) {
   if (lzx) lzx->length = out_bytes;
 }
 
-int lzx_decompress(struct lzx_stream *lzx, off_t out_bytes) {
+int lzx_decompress(struct lzx_stream *lzx, uint32_t out_bytes) {
   /* bitstream reading and huffman variables */
   register unsigned int bit_buffer;
   register int bits_left, i=0;
@@ -1107,7 +1115,7 @@ int lzx_decompress(struct lzx_stream *lzx, off_t out_bytes) {
   unsigned int R0, R1, R2;
 
   /* easy answers */
-  if (!lzx || (out_bytes < 0)) return CL_ENULLARG;
+  if (!lzx) return CL_ENULLARG;
   if (lzx->error) return lzx->error;
 
   /* flush out any stored-up bytes before we begin */
@@ -1612,7 +1620,7 @@ void lzx_free(struct lzx_stream *lzx) {
 } while (0)
 
 static int qtm_read_input(struct qtm_stream *qtm) {
-  int nread = qtm->read_cb ? qtm->read_cb(qtm->file, &qtm->inbuf[0], (int)qtm->inbuf_size) : cli_readn(qtm->fd, &qtm->inbuf[0], (int)qtm->inbuf_size);
+  int nread = qtm->read_cb(qtm->file, &qtm->inbuf[0], (int)qtm->inbuf_size);
   if (nread < 0) {
     if (qtm->file->error == CL_BREAK)
       return qtm->error = CL_BREAK;
@@ -1738,7 +1746,7 @@ static void qtm_init_model(struct qtm_model *model,
 
 /*-------- main Quantum code --------*/
 
-struct qtm_stream *qtm_init(int fd, int ofd,
+struct qtm_stream *qtm_init(int ofd,
 			      int window_bits, int input_buffer_size,
 			      struct cab_file *file,
 			      int (*read_cb)(struct cab_file *, unsigned char *, int))
@@ -1777,19 +1785,20 @@ struct qtm_stream *qtm_init(int fd, int ofd,
   /* allocate decompression window and input buffer */
   qtm->window = cli_malloc((size_t) window_size);
   if (!qtm->window) {
+      cli_errmsg("qtm_init: Unable to allocate decompression window\n");
     free(qtm);
     return NULL;
   }
 
   qtm->inbuf  = cli_malloc((size_t) input_buffer_size);
   if (!qtm->inbuf) {
+      cli_errmsg("qtm_init: Unable to allocate input buffer\n");
     free(qtm->window);
     free(qtm);
     return NULL;
   }
 
   /* initialise decompression state */
-  qtm->fd	   = fd;
   qtm->ofd	   = ofd;
   qtm->wflag	   = 1;
   qtm->inbuf_size  = input_buffer_size;
@@ -1827,7 +1836,7 @@ struct qtm_stream *qtm_init(int fd, int ofd,
   return qtm;
 }
 
-int qtm_decompress(struct qtm_stream *qtm, off_t out_bytes) {
+int qtm_decompress(struct qtm_stream *qtm, uint32_t out_bytes) {
   unsigned int frame_start, frame_end, window_posn, match_offset, range;
   unsigned char *window, *i_ptr, *i_end, *runsrc, *rundest;
   int i, j, selector, extra, sym, match_length, ret;
@@ -1838,7 +1847,7 @@ int qtm_decompress(struct qtm_stream *qtm, off_t out_bytes) {
   unsigned char bits_needed, bit_run;
 
   /* easy answers */
-  if (!qtm || (out_bytes < 0)) return CL_ENULLARG;
+  if (!qtm) return CL_ENULLARG;
   if (qtm->error) return qtm->error;
 
   /* flush out any stored-up bytes before we begin */
@@ -1876,6 +1885,10 @@ int qtm_decompress(struct qtm_stream *qtm, off_t out_bytes) {
     if ((frame_start + QTM_FRAME_SIZE) < frame_end) {
       frame_end = frame_start + QTM_FRAME_SIZE;
     }
+    if (frame_end < window_posn) {
+	cli_dbgmsg("qtm_decompress: window position beyond end of frame\n");
+	return qtm->error = CL_EFORMAT;
+    }
 
     while (window_posn < frame_end) {
       QTM_GET_SYMBOL(qtm->model7, selector);
