@@ -70,7 +70,7 @@ struct nsis_st {
   struct CLI_LZMA lz;
 /*   z_stream z; */
   nsis_z_stream z;
-  unsigned char *freeme;
+  const unsigned char *freeme;
   fmap_t *map;
   char ofn[1024];
 };
@@ -183,7 +183,7 @@ static int nsis_decomp(struct nsis_st *n) {
 }
 
 static int nsis_unpack_next(struct nsis_st *n, cli_ctx *ctx) {
-  unsigned char *ibuf;
+  const unsigned char *ibuf;
   uint32_t size, loops;
   int ret, gotsome=0;
   unsigned char obuf[BUFSIZ];
@@ -258,7 +258,7 @@ static int nsis_unpack_next(struct nsis_st *n, cli_ctx *ctx) {
       }
       
       n->nsis.avail_in = size;
-      n->nsis.next_in = ibuf;
+      n->nsis.next_in = (void*)ibuf;
       n->nsis.next_out = obuf;
       n->nsis.avail_out = BUFSIZ;
       loops=0;
@@ -325,7 +325,7 @@ static int nsis_unpack_next(struct nsis_st *n, cli_ctx *ctx) {
 	close(n->ofd);
 	return CL_EREAD;
       }
-      n->nsis.next_in = n->freeme;
+      n->nsis.next_in = (void*)n->freeme;
       n->nsis.avail_in = n->asz;
     }
 
@@ -488,7 +488,7 @@ static int cli_nsis_unpack(struct nsis_st *n, cli_ctx *ctx) {
 }
 
 
-int cli_scannulsft(int desc, cli_ctx *ctx, off_t offset) {
+int cli_scannulsft(cli_ctx *ctx, off_t offset) {
         int ret;
 	struct nsis_st nsist;
 
@@ -505,18 +505,18 @@ int cli_scannulsft(int desc, cli_ctx *ctx, off_t offset) {
 	return CL_ETMPDIR;
     }
     
-    if(!(nsist.map = fmap(desc, 0, 0))) {
-	cli_errmsg("scannulsft: fmap failed\n");
-	return CL_EMEM;
-    }
-
+    nsist.map = *ctx->fmap;
     if(ctx->engine->keeptmp) cli_dbgmsg("NSIS: Extracting files to %s\n", nsist.dir);
 
     do {
         ret = cli_nsis_unpack(&nsist, ctx);
 	if (ret == CL_SUCCESS) {
 	  cli_dbgmsg("NSIS: Successully extracted file #%u\n", nsist.fno);
-	  lseek(nsist.ofd, 0, SEEK_SET);
+	  if (lseek(nsist.ofd, 0, SEEK_SET) == -1) {
+          cli_dbgmsg("NSIS: call to lseek() failed\n");
+          free(nsist.dir);
+        return CL_ESEEK;
+      }
 	  if(nsist.fno == 1)
 	    ret=cli_scandesc(nsist.ofd, ctx, 0, 0, NULL, AC_SCAN_VIR, NULL);
 	  else
@@ -533,7 +533,6 @@ int cli_scannulsft(int desc, cli_ctx *ctx, off_t offset) {
 	ret = CL_CLEAN;
 
     nsis_shutdown(&nsist);
-    funmap(nsist.map);
 
     if(!ctx->engine->keeptmp)
         cli_rmdirs(nsist.dir);
