@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2007-2008 Sourcefire, Inc.
+ *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, Trog
  *
@@ -119,6 +119,7 @@ void cli_logg_unsetup(void)
 }
 #endif
 uint8_t cli_debug_flag = 0;
+uint8_t cli_always_gen_section_hash = 0;
 
 static void fputs_callback(enum cl_msg severity, const char *fullmsg, const char *msg, void *context)
 {
@@ -194,8 +195,8 @@ void *cli_malloc(size_t size)
     alloc = malloc(size);
 
     if(!alloc) {
-	cli_errmsg("cli_malloc(): Can't allocate memory (%lu bytes).\n", (unsigned long int) size);
 	perror("malloc_problem");
+	cli_errmsg("cli_malloc(): Can't allocate memory (%lu bytes).\n", (unsigned long int) size);
 	return NULL;
     } else return alloc;
 }
@@ -214,8 +215,8 @@ void *cli_calloc(size_t nmemb, size_t size)
     alloc = calloc(nmemb, size);
 
     if(!alloc) {
-	cli_errmsg("cli_calloc(): Can't allocate memory (%lu bytes).\n", (unsigned long int) (nmemb * size));
 	perror("calloc_problem");
+	cli_errmsg("cli_calloc(): Can't allocate memory (%lu bytes).\n", (unsigned long int) (nmemb * size));
 	return NULL;
     } else return alloc;
 }
@@ -233,8 +234,8 @@ void *cli_realloc(void *ptr, size_t size)
     alloc = realloc(ptr, size);
 
     if(!alloc) {
-	cli_errmsg("cli_realloc(): Can't re-allocate memory to %lu bytes.\n", (unsigned long int) size);
 	perror("realloc_problem");
+	cli_errmsg("cli_realloc(): Can't re-allocate memory to %lu bytes.\n", (unsigned long int) size);
 	return NULL;
     } else return alloc;
 }
@@ -252,8 +253,8 @@ void *cli_realloc2(void *ptr, size_t size)
     alloc = realloc(ptr, size);
 
     if(!alloc) {
-	cli_errmsg("cli_realloc2(): Can't re-allocate memory to %lu bytes.\n", (unsigned long int) size);
 	perror("realloc_problem");
+	cli_errmsg("cli_realloc2(): Can't re-allocate memory to %lu bytes.\n", (unsigned long int) size);
 	if(ptr)
 	    free(ptr);
 	return NULL;
@@ -273,8 +274,8 @@ char *cli_strdup(const char *s)
     alloc = strdup(s);
 
     if(!alloc) {
-        cli_errmsg("cli_strdup(): Can't allocate memory (%u bytes).\n", (unsigned int) strlen(s));
         perror("strdup_problem");
+        cli_errmsg("cli_strdup(): Can't allocate memory (%u bytes).\n", (unsigned int) strlen(s));
         return NULL;
     }
 
@@ -520,7 +521,7 @@ static int get_filetype(const char *fname, int flags, int need_stat,
     }
 
     if (need_stat) {
-	if (STAT(fname, statbuf) == -1)
+	if (CLAMSTAT(fname, statbuf) == -1)
 	    return -1;
 	stated = 1;
     }
@@ -688,6 +689,7 @@ static int cli_ftw_dir(const char *dirname, int flags, int maxdepth, cli_ftw_cb
 		ret = callback(NULL, NULL, dirname, error_mem, data);
 		if (ret != CL_SUCCESS)
 		    break;
+		continue; /* have to skip this one if continuing after error */
 	    }
             if(!strcmp(dirname, PATHSEP))
 		sprintf(fname, PATHSEP"%s", dent->d_name);
@@ -810,6 +812,7 @@ const char* cli_strerror(int errnum, char *buf, size_t len)
 #endif
     err = strerror(errnum);
     strncpy(buf, err, len);
+    buf[len-1] = '\0'; /* just in case */
 # ifdef CL_THREAD_SAFE
     pthread_mutex_unlock(&cli_strerror_mutex);
 #endif
