@@ -1,5 +1,7 @@
 /*
- *  Copyright (C) 2006 aCaB <acab@clamav.net>
+ *  Copyright (C) 2007-2008 Sourcefire, Inc.
+ *
+ *  Authors: Alberto Wu, Michal 'GiM' Spadlinski
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -16,7 +18,6 @@
  *  MA 02110-1301, USA.
  */
 
-
 #if HAVE_CONFIG_H
 #include "clamav-config.h"
 #endif
@@ -26,7 +27,7 @@
 #include "pe.h"
 #include "packlibs.h"
 
-static int doubledl(char **scur, uint8_t *mydlptr, char *buffer, uint32_t buffersize)
+static int doubledl(const char **scur, uint8_t *mydlptr, const char *buffer, uint32_t buffersize)
 {
   unsigned char mydl = *mydlptr;
   unsigned char olddl = mydl;
@@ -44,10 +45,11 @@ static int doubledl(char **scur, uint8_t *mydlptr, char *buffer, uint32_t buffer
 }
 
 
-int cli_unfsg(char *source, char *dest, int ssize, int dsize, char **endsrc, char **enddst) {
+int cli_unfsg(const char *source, char *dest, int ssize, int dsize, const char **endsrc, char **enddst) {
   uint8_t mydl=0x80;
   uint32_t backbytes, backsize, oldback = 0;
-  char *csrc = source, *cdst = dest;
+  const char *csrc = source;
+  char *cdst = dest;
   int oob, lostbit = 1;
 
   if (ssize<=0 || dsize<=0) return -1;
@@ -169,10 +171,11 @@ int cli_unfsg(char *source, char *dest, int ssize, int dsize, char **endsrc, cha
   return 0;
 }
 
-int unmew(char *source, char *dest, int ssize, int dsize, char **endsrc, char **enddst) {
+int unmew(const char *source, char *dest, int ssize, int dsize, const char **endsrc, char **enddst) {
   uint8_t mydl=0x80;
   uint32_t myeax_backbytes, myecx_backsize, oldback = 0;
-  char *csrc = source, *cdst = dest;
+  const char *csrc = source;
+  char *cdst = dest;
   int oob, lostbit = 1;
 
   *cdst++=*csrc++;
@@ -278,7 +281,7 @@ int unmew(char *source, char *dest, int ssize, int dsize, char **endsrc, char **
       }
       if (!CLI_ISCONTAINED(dest, dsize, cdst, myecx_backsize) || !CLI_ISCONTAINED(dest, dsize, cdst-myeax_backbytes, myecx_backsize))
       {
-	cli_dbgmsg("MEW: rete: %d %d %d %d %d || %d %d %d %d %d\n", dest, dsize, cdst, myecx_backsize,
+	cli_dbgmsg("MEW: rete: %p %d %p %d %d || %p %d %p %d %d\n", dest, dsize, cdst, myecx_backsize,
 			CLI_ISCONTAINED(dest, dsize, cdst, myecx_backsize),
 			dest, dsize, cdst-myeax_backbytes, myecx_backsize,
       			CLI_ISCONTAINED(dest, dsize, cdst-myeax_backbytes, myecx_backsize) );
@@ -293,7 +296,7 @@ int unmew(char *source, char *dest, int ssize, int dsize, char **endsrc, char **
       /* 15d */
       if (cdst < dest || cdst >= dest+dsize || csrc < source || csrc >= source+ssize)
       {
-	cli_dbgmsg("MEW: retf %08x %08x+%08x=%08x, %08x %08x+%08x=%08x\n",
+	cli_dbgmsg("MEW: retf %p %p+%08x=%p, %p %p+%08x=%p\n",
 			cdst, dest, dsize, dest+dsize, csrc, source, ssize, source+ssize);
 	return -1;
       }
