@@ -1,5 +1,7 @@
 /*
- *  Copyright (C) 2005 aCaB <acab@clamav.net>
+ *  Copyright (C) 2007-2008 Sourcefire, Inc.
+ *
+ *  Authors: Alberto Wu, Michal 'GiM' Spadlinski
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -26,8 +28,8 @@
 #include "cltypes.h"
 #include "rebuildpe.h"
 
-int cli_unfsg(char *, char *, int, int, char **, char **);
+int cli_unfsg(const char *, char *, int, int, const char **, char **);
 
-int unmew(char *, char *, int, int, char **, char **);
+int unmew(const char *, char *, int, int, const char **, char **);
 
 #endif
