@@ -1589,7 +1589,7 @@ static int cli_loadftm(FILE *fs, struct cl_engine *engine, unsigned int options,
 	struct cli_ftype *new;
 	cli_file_t rtype, type;
 	int ret;
-
+	int magictype;
 
     if((ret = cli_initroots(engine, options)))
 	return ret;
@@ -1650,11 +1650,12 @@ static int cli_loadftm(FILE *fs, struct cl_engine *engine, unsigned int options,
 	    break;
 	}
 
-	if(atoi(tokens[0]) == 1) { /* A-C */
+        magictype = atoi(tokens[0]);
+	if(magictype == 1) { /* A-C */
 	    if((ret = cli_parse_add(engine->root[0], tokens[3], tokens[2], rtype, type, tokens[1], 0, NULL, options)))
 		break;
 
-	} else if(atoi(tokens[0]) == 0) { /* memcmp() */
+	} else if ((magictype == 0) || (magictype == 4)) { /* memcmp() */
 	    if(!cli_isnumber(tokens[1])) {
 		cli_errmsg("cli_loadftm: Invalid offset\n");
 		ret = CL_EMALFDB;
@@ -1682,9 +1683,15 @@ static int cli_loadftm(FILE *fs, struct cl_engine *engine, unsigned int options,
 		ret = CL_EMEM;
 		break;
 	    }
-	    new->next = engine->ftypes;
-	    engine->ftypes = new;
-
+            /* files => ftypes, partitions => ptypes */
+	    if(magictype == 4) {
+		new->next = engine->ptypes;
+		engine->ptypes = new;
+	    }
+	    else {
+		new->next = engine->ftypes;
+		engine->ftypes = new;
+            }
 	} else {
 	    cli_dbgmsg("cli_loadftm: Unsupported mode %u\n", atoi(tokens[0]));
 	    continue;
@@ -2740,7 +2747,8 @@ static int cli_loaddbdir(const char *dirname, struct cl_engine *engine, unsigned
 	} result;
 #endif
 	char *dbfile;
-	int ret = CL_EOPEN, have_cld;
+	int ret = CL_EOPEN, have_cld, ends_with_sep = 0;
+	size_t dirname_len;
 	struct cl_cvd *daily_cld, *daily_cvd;
 
 
@@ -2751,6 +2759,14 @@ static int cli_loaddbdir(const char *dirname, struct cl_engine *engine, unsigned
         return CL_EOPEN;
     }
 
+    dirname_len = strlen(dirname);
+    if(dirname_len >= strlen(PATHSEP)) {
+        if(strcmp(dirname + dirname_len - strlen(PATHSEP), PATHSEP) == 0) {
+            cli_dbgmsg("cli_loaddbdir(): dirname ends with separator\n");
+            ends_with_sep = 1;
+        }
+    }
+
     /* first round - load .ign and .ign2 files */
 #ifdef HAVE_READDIR_R_3
     while(!readdir_r(dd, &result.d, &dent) && dent) {
@@ -2762,13 +2778,16 @@ static int cli_loaddbdir(const char *dirname, struct cl_engine *engine, unsigned
 	if(dent->d_ino)
 	{
 	    if(cli_strbcasestr(dent->d_name, ".ign") || cli_strbcasestr(dent->d_name, ".ign2")) {
-		dbfile = (char *) cli_malloc(strlen(dent->d_name) + strlen(dirname) + 2);
+		dbfile = (char *) cli_malloc(strlen(dent->d_name) + dirname_len + 2);
 		if(!dbfile) {
 		    cli_errmsg("cli_loaddbdir(): dbfile == NULL\n");
 		    closedir(dd);
 		    return CL_EMEM;
 		}
-		sprintf(dbfile, "%s"PATHSEP"%s", dirname, dent->d_name);
+		if(ends_with_sep)
+		    sprintf(dbfile, "%s%s", dirname, dent->d_name);
+		else
+                    sprintf(dbfile, "%s"PATHSEP"%s", dirname, dent->d_name);
 		ret = cli_load(dbfile, engine, signo, options, NULL);
 		if(ret) {
 		    cli_errmsg("cli_loaddbdir(): error loading database %s\n", dbfile);
@@ -2782,14 +2801,17 @@ static int cli_loaddbdir(const char *dirname, struct cl_engine *engine, unsigned
     }
 
     /* the daily db must be loaded before main */
-    dbfile = (char *) cli_malloc(strlen(dirname) + 20);
+    dbfile = (char *) cli_malloc(dirname_len + 20);
     if(!dbfile) {
 	closedir(dd);
     cli_errmsg("cli_loaddbdir: Can't allocate memory for dbfile\n");
 	return CL_EMEM;
     }
 
-    sprintf(dbfile, "%s"PATHSEP"daily.cld", dirname);
+    if(ends_with_sep)
+        sprintf(dbfile, "%sdaily.cld", dirname);
+    else
+        sprintf(dbfile, "%s"PATHSEP"daily.cld", dirname);
     have_cld = !access(dbfile, R_OK);
     if(have_cld) {
 	daily_cld = cl_cvdhead(dbfile);
@@ -2800,7 +2822,10 @@ static int cli_loaddbdir(const char *dirname, struct cl_engine *engine, unsigned
 	    return CL_EMALFDB;
 	}
     }
-    sprintf(dbfile, "%s"PATHSEP"daily.cvd", dirname); 
+    if(ends_with_sep)
+        sprintf(dbfile, "%sdaily.cvd", dirname);
+    else
+        sprintf(dbfile, "%s"PATHSEP"daily.cvd", dirname);
     if(!access(dbfile, R_OK)) {
 	if(have_cld) {
 	    daily_cvd = cl_cvdhead(dbfile);
@@ -2811,12 +2836,19 @@ static int cli_loaddbdir(const char *dirname, struct cl_engine *engine, unsigned
 		closedir(dd);
 		return CL_EMALFDB;
 	    }
-	    if(daily_cld->version > daily_cvd->version)
-		sprintf(dbfile, "%s"PATHSEP"daily.cld", dirname);
+	    if(daily_cld->version > daily_cvd->version) {
+		if(ends_with_sep)
+                    sprintf(dbfile, "%sdaily.cld", dirname);
+		else
+                    sprintf(dbfile, "%s"PATHSEP"daily.cld", dirname);
+	    }
 	    cl_cvdfree(daily_cvd);
 	}
     } else {
-	sprintf(dbfile, "%s"PATHSEP"daily.cld", dirname);
+	if(ends_with_sep)
+	    sprintf(dbfile, "%sdaily.cld", dirname);
+	else
+	    sprintf(dbfile, "%s"PATHSEP"daily.cld", dirname);
     }
     if(have_cld)
 	cl_cvdfree(daily_cld);
@@ -2828,7 +2860,10 @@ static int cli_loaddbdir(const char *dirname, struct cl_engine *engine, unsigned
     }
 
     /* try to load local.gdb next */
-    sprintf(dbfile, "%s"PATHSEP"local.gdb", dirname);
+    if(ends_with_sep)
+        sprintf(dbfile, "%slocal.gdb", dirname);
+    else
+        sprintf(dbfile, "%s"PATHSEP"local.gdb", dirname);
     if(!access(dbfile, R_OK) && (ret = cli_load(dbfile, engine, signo, options, NULL))) {
 	free(dbfile);
 	closedir(dd);
@@ -2836,7 +2871,10 @@ static int cli_loaddbdir(const char *dirname, struct cl_engine *engine, unsigned
     }
 
     /* check for and load daily.cfg */
-    sprintf(dbfile, "%s"PATHSEP"daily.cfg", dirname);
+    if(ends_with_sep)
+        sprintf(dbfile, "%sdaily.cfg", dirname);
+    else
+        sprintf(dbfile, "%s"PATHSEP"daily.cfg", dirname);
     if(!access(dbfile, R_OK) && (ret = cli_load(dbfile, engine, signo, options, NULL))) {
 	free(dbfile);
 	closedir(dd);
@@ -2861,13 +2899,16 @@ static int cli_loaddbdir(const char *dirname, struct cl_engine *engine, unsigned
 		    continue;
 		}
 
-		dbfile = (char *) cli_malloc(strlen(dent->d_name) + strlen(dirname) + 2);
+		dbfile = (char *) cli_malloc(strlen(dent->d_name) + dirname_len + 2);
 		if(!dbfile) {
 		    cli_errmsg("cli_loaddbdir(): dbfile == NULL\n");
 		    closedir(dd);
 		    return CL_EMEM;
 		}
-		sprintf(dbfile, "%s"PATHSEP"%s", dirname, dent->d_name);
+                if(ends_with_sep)
+		    sprintf(dbfile, "%s%s", dirname, dent->d_name);
+                else
+		    sprintf(dbfile, "%s"PATHSEP"%s", dirname, dent->d_name);
 		ret = cli_load(dbfile, engine, signo, options, NULL);
 		if(ret) {
 		    cli_errmsg("cli_loaddbdir(): error loading database %s\n", dbfile);
@@ -2901,8 +2942,37 @@ int cl_load(const char *path, struct cl_engine *engine, unsigned int *signo, uns
 	return CL_EARG;
     }
 
-    if(STAT(path, &sb) == -1) {
-        cli_errmsg("cl_load(): Can't get status of %s\n", path);
+    if(CLAMSTAT(path, &sb) == -1) {
+        switch (errno) {
+#if defined(EACCES)
+            case EACCES:
+                cli_errmsg("cl_load(): Access denied for path: %s\n", path);
+                break;
+#endif
+#if defined(ENOENT)
+            case ENOENT:
+                cli_errmsg("cl_load(): No such file or directory: %s\n", path);
+                break;
+#endif
+#if defined(ELOOP)
+            case ELOOP:
+                cli_errmsg("cl_load(): Too many symbolic links encountered in path: %s\n", path);
+                break;
+#endif
+#if defined(EOVERFLOW)
+            case EOVERFLOW:
+                cli_errmsg("cl_load(): File size is too large to be recognized. Path: %s\n", path);
+                break;
+#endif
+#if defined(EIO)
+            case EIO:
+                cli_errmsg("cl_load(): An I/O error occurred while reading from path: %s\n", path);
+                break;
+#endif
+            default:
+                cli_errmsg("cl_load: Can't get status of: %s\n", path);
+                break;
+        }
         return CL_ESTAT;
     }
 
@@ -3021,7 +3091,7 @@ int cl_statinidir(const char *dirname, struct cl_stat *dbstat)
 
 		strcpy(dbstat->statdname[dbstat->entries - 1], dent->d_name);
 #endif
-		STAT(fname, &dbstat->stattab[dbstat->entries - 1]);
+		CLAMSTAT(fname, &dbstat->stattab[dbstat->entries - 1]);
 		free(fname);
 	    }
 	}
@@ -3076,7 +3146,7 @@ int cl_statchkdir(const struct cl_stat *dbstat)
 		}
 
 		sprintf(fname, "%s"PATHSEP"%s", dbstat->dir, dent->d_name);
-		STAT(fname, &sb);
+		CLAMSTAT(fname, &sb);
 		free(fname);
 
 		found = 0;
@@ -3426,7 +3496,7 @@ int cl_countsigs(const char *path, unsigned int countoptions, unsigned int *sigs
     if(!sigs)
 	return CL_ENULLARG;
 
-    if(STAT(path, &sb) == -1) {
+    if(CLAMSTAT(path, &sb) == -1) {
 	cli_errmsg("cl_countsigs: Can't stat %s\n", path);
 	return CL_ESTAT;
     }
