@@ -57,7 +57,10 @@
 	cli_strbcasestr(ext, ".cfg")   ||	\
 	cli_strbcasestr(ext, ".cvd")   ||	\
 	cli_strbcasestr(ext, ".cld")   ||	\
+	cli_strbcasestr(ext, ".cud")   ||	\
 	cli_strbcasestr(ext, ".cdb")   ||	\
+	cli_strbcasestr(ext, ".cat")   ||	\
+	cli_strbcasestr(ext, ".crb")   ||	\
 	cli_strbcasestr(ext, ".idb")		\
     )
 
