@@ -214,8 +214,10 @@ static int load_actions(table_t* t)
 static int rtf_object_begin(struct rtf_state* state,cli_ctx* ctx,const char* tmpdir)
 {
 	struct rtf_object_data* data = cli_malloc(sizeof(*data));
-	if(!data)
+	if(!data) {
+        cli_errmsg("rtf_object_begin: Unable to allocate memory for object data\n");
 		return CL_EMEM;
+    }
 	data->fd = -1;
 	data->partial = 0;
 	data->has_partial = 0;
@@ -246,8 +248,8 @@ static int decode_and_scan(struct rtf_object_data* data, cli_ctx* ctx)
 	close(data->fd);
 	data->fd = -1;
 	if(data->name) {
-		if(!cli_leavetemps_flag)
-			unlink(data->name);
+		if(!ctx->engine->keeptmp)
+			if(cli_unlink(data->name)) ret = CL_EUNLINK;
 		free(data->name);
 		data->name = NULL;
 	}
@@ -329,6 +331,7 @@ static int rtf_object_process(struct rtf_state* state, const unsigned char* inpu
 							    else
 								    data->desc_name = cli_malloc(data->desc_len+1);
 							    if(!data->desc_name) {
+                                    cli_errmsg("rtf_object_process: Unable to allocate memory for data->desc_name\n");
 								    return CL_EMEM;
 							    }
 							    data->internal_state = WAIT_DESC;
@@ -409,7 +412,7 @@ static int rtf_object_process(struct rtf_state* state, const unsigned char* inpu
 							    data->bread = 1;/* flag to indicate this needs to be scanned with cli_decode_ole_object*/
 							    cli_writeint32(out,data->desc_len);
 							    if(cli_writen(data->fd,out,4)!=4)
-								    return CL_EIO; 
+								    return CL_EWRITE; 
 							}
 							else
 								data->bread = 2;
@@ -417,7 +420,7 @@ static int rtf_object_process(struct rtf_state* state, const unsigned char* inpu
 
 						data->desc_len -= out_want;
 						if(cli_writen(data->fd,out_data,out_want) != out_want) {
-							return CL_EIO;
+							return CL_EWRITE;
 						}
 						out_data += out_want;
 						out_cnt  -= out_want;
@@ -493,24 +496,23 @@ static void cleanup_stack(struct stack* stack,struct rtf_state* state,cli_ctx* c
 		state.cb_end(&state,ctx);\
 	tableDestroy(actiontable);\
 	cleanup_stack(&stack,&state,ctx);\
-	free(buff);\
-        if(!cli_leavetemps_flag)\
+        if(!ctx->engine->keeptmp)\
 		cli_rmdirs(tempname);\
 	free(tempname);\
 	free(stack.states);
 
-int cli_scanrtf(int desc, cli_ctx *ctx)
+int cli_scanrtf(cli_ctx *ctx)
 {
 	char* tempname;
 	const unsigned char* ptr;
 	const unsigned char* ptr_end;
-	unsigned char* buff;
 	int ret = CL_CLEAN;
 	struct rtf_state state;
 	struct stack stack;
-	ssize_t bread;
+	size_t bread;
 	table_t* actiontable;
 	uint8_t main_symbols[256];
+	size_t offset = 0;
 
 	cli_dbgmsg("in cli_scanrtf()\n");
 
@@ -525,22 +527,17 @@ int cli_scanrtf(int desc, cli_ctx *ctx)
 	stack.warned = 0;
 	stack.states = cli_malloc(stack.stack_size*sizeof(*stack.states));
 
-	if(!stack.states)
+	if(!stack.states) {
+        cli_errmsg("ScanRTF: Unable to allocate memory for stack states\n");
 		return CL_EMEM;
+    }
 
-	buff = cli_malloc(BUFF_SIZE);
-	if(!buff) {
-		free(stack.states);
-		return CL_EMEM;
-	}
-
-	if(!(tempname = cli_gentemp(NULL)))
+	if(!(tempname = cli_gentemp(ctx->engine->tmpdir)))
 	    return CL_EMEM;
 
 	if(mkdir(tempname, 0700)) {
 	    	cli_dbgmsg("ScanRTF -> Can't create temporary directory %s\n", tempname);
 		free(stack.states);
-		free(buff);
 		free(tempname);
 		return CL_ETMPDIR;
 	}
@@ -549,8 +546,7 @@ int cli_scanrtf(int desc, cli_ctx *ctx)
 	if((ret = load_actions(actiontable))) {
 		cli_dbgmsg("RTF: Unable to load rtf action table\n");
 		free(stack.states);
-		free(buff);
-		if(!cli_leavetemps_flag)
+		if(!ctx->engine->keeptmp)
 			cli_rmdirs(tempname);
 		free(tempname);
 		tableDestroy(actiontable);
@@ -559,10 +555,9 @@ int cli_scanrtf(int desc, cli_ctx *ctx)
 
 	init_rtf_state(&state);
 
-	while(( bread = cli_readn(desc, buff, BUFF_SIZE) ) > 0) {
-		ptr = buff;
-		ptr_end = buff + bread;
-		while(ptr < ptr_end) {
+	for (offset = 0; (ptr = fmap_need_off_once_len(*ctx->fmap, offset, BUFF_SIZE, &bread)) && bread; offset += bread) {
+	    ptr_end = ptr + bread;
+	    while(ptr < ptr_end) {
 			switch(state.parse_state) {
 				case PARSE_MAIN: 
 					switch(*ptr++) {
@@ -617,7 +612,7 @@ int cli_scanrtf(int desc, cli_ctx *ctx)
 							}
 					}
 					break;
-				case PARSE_CONTROL_:					
+				case PARSE_CONTROL_:
 					if(isalpha(*ptr))  {
 						state.parse_state = PARSE_CONTROL_WORD;
 						state.controlword_cnt = 0;
